@@ -1127,7 +1127,7 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
         s.setPayeeId(payeeid);
         d->setupPrice(s, splitAccount, d->m_account, statementTransactionUnderImport.m_datePosted);
         transactionUnderImport.addSplit(s);
-        file->addVATSplit(transactionUnderImport, d->m_account, splitAccount, statementTransactionUnderImport.m_amount);
+
       } else if (statementTransactionUnderImport.m_listSplits.isEmpty() && !d->m_skipCategoryMatching) {
         MyMoneyTransactionFilter filter(thisaccount.id());
         filter.addPayee(payeeid);
@@ -1137,30 +1137,34 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
           MyMoneyTransaction t_old = list.last();
 
           // if there is more than one matching transaction, try to be a little
-          // smart about which one we take.  for now, we'll see if there's one
-          // with the same VALUE as our imported transaction, and if so take that one.
+          // smart about which one we use.  we scan them all and check if
+          // we find an exact match or use the one with the closest value
+
           if (list.count() > 1) {
             QList<MyMoneyTransaction>::ConstIterator it_trans = list.constEnd();
-            if (it_trans != list.constBegin())
+            MyMoneyMoney minDiff;
+            do {
               --it_trans;
-            while (it_trans != list.constBegin()) {
               MyMoneySplit s = (*it_trans).splitByAccount(thisaccount.id());
               if (s.value() == s1.value()) {
+                // in case of an exact match, we won't get better and we can stop.
                 // keep searching if this transaction references a closed account
                 if (!MyMoneyFile::instance()->referencesClosedAccount(*it_trans)) {
                   t_old = *it_trans;
                   break;
                 }
+              } else {
+                MyMoneyMoney newDiff = (s.value() - s1.value()).abs();
+                if (newDiff < minDiff || minDiff.isZero()) {
+                  // keep it if it matches better than the current match
+                  // but only if it does not reference a closed account
+                  if (!MyMoneyFile::instance()->referencesClosedAccount(*it_trans)) {
+                    minDiff = newDiff;
+                    t_old = *it_trans;
+                  }
+                }
               }
-              --it_trans;
-            }
-            // check constBegin, just in case
-            if (it_trans == list.constBegin()) {
-              MyMoneySplit s = (*it_trans).splitByAccount(thisaccount.id());
-              if (s.value() == s1.value()) {
-                t_old = *it_trans;
-              }
-            }
+            } while (it_trans != list.constBegin());
           }
 
           // Only copy the splits if the transaction found does not reference a closed account
@@ -1175,6 +1179,12 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
                 s.setBankID(QString());
                 s.removeMatch();
 
+                // in case the old transaction has two splits
+                // we simply inverse the amount of the current
+                // transaction found in s1. In other cases (more
+                // than two splits we copy all splits and don't
+                // modify the splits. This may lead to unbalanced
+                // transactions which the user has to fix manually
                 if (t_old.splits().count() == 2) {
                   s.setShares(-s1.shares());
                   s.setValue(-s1.value());
@@ -1229,6 +1239,9 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
 
   transactionUnderImport.addSplit(s1);
 
+  // check if we need to add/update a VAT assignment
+  file->updateVAT(transactionUnderImport);
+
   if ((statementTransactionUnderImport.m_eAction != eMyMoney::Transaction::Action::ReinvestDividend) && (statementTransactionUnderImport.m_eAction != eMyMoney::Transaction::Action::CashDividend) && (statementTransactionUnderImport.m_eAction != eMyMoney::Transaction::Action::Interest)
      ) {
     //******************************************
