@@ -1,5 +1,5 @@
 /*
- * Copyright 2007-2018  Thomas Baumgart <tbaumgart@kde.org>
+ * Copyright 2007-2019  Thomas Baumgart <tbaumgart@kde.org>
  * Copyright 2017-2018  Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -287,10 +287,13 @@ void Activity::setWidgetVisibility(const QStringList& widgetIds, bool visible) c
   for (QStringList::const_iterator it_w = widgetIds.constBegin(); it_w != widgetIds.constEnd(); ++it_w) {
     auto w = haveWidget(*it_w);
     if (w) {
-      if (visible) {
-        w->show();
+      // in case we hit a category with a split button,
+      // we need to manipulate the enclosing QFrame
+      auto cat = dynamic_cast<KMyMoneyCategory*>(w);
+      if (cat && cat->splitButton()) {
+        cat->parentWidget()->setVisible(visible);
       } else {
-        w->hide();
+        w->setVisible(visible);
       }
     }
   }
@@ -331,11 +334,12 @@ eMyMoney::Split::InvestmentTransactionType Buy::type() const
 
 void Buy::showWidgets() const
 {
-  static const QStringList visibleWidgetIds = QStringList() << "asset-account" << "shares" << "price" << "total" << "interest-account" << "fee-account";
+  static const QStringList visibleWidgetIds = QStringList() << "asset-account" << "shares" << "price" << "total" << "fee-account" << "fee-amount" << "interest-account" << "interest-amount";
   setWidgetVisibility(visibleWidgetIds, true);
 
   setLabelText("interest-amount-label", i18n("Interest"));
   setLabelText("interest-label", i18n("Interest"));
+  setLabelText("fee-amount-label", i18n("Fees"));
   setLabelText("fee-label", i18n("Fees"));
   setLabelText("asset-label", i18n("Account"));
   setLabelText("shares-label", i18n("Shares"));
@@ -432,7 +436,7 @@ eMyMoney::Split::InvestmentTransactionType Sell::type() const
 void Sell::showWidgets() const
 {
   Q_D(const Activity);
-  static const QStringList visibleWidgetIds = QStringList() << "asset-account" << "interest-amount" << "shares" << "price" << "total" << "interest-account" << "fee-account";
+  static const QStringList visibleWidgetIds = QStringList() << "asset-account" << "interest-amount" << "fee-amount" << "shares" << "price" << "total" << "interest-account" << "fee-account";
   setWidgetVisibility(visibleWidgetIds, true);
 
   if (auto shareEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares")))
@@ -440,6 +444,7 @@ void Sell::showWidgets() const
 
   setLabelText("interest-amount-label", i18n("Interest"));
   setLabelText("interest-label", i18n("Interest"));
+  setLabelText("fee-amount-label", i18n("Fees"));
   setLabelText("fee-label", i18n("Fees"));
   setLabelText("asset-label", i18n("Account"));
   setLabelText("shares-label", i18n("Shares"));
@@ -450,12 +455,23 @@ void Sell::showWidgets() const
 
 bool Sell::isComplete(QString& reason) const
 {
+  Q_D(const Activity);
+
   auto rc = Activity::isComplete(reason);
-  rc &= haveAssetAccount();
   rc &= haveFees(true);
   rc &= haveInterest(true);
   rc &= haveShares();
   rc &= havePrice();
+
+  // Allow a sell operation to be saved without specifying a brokerage
+  // account, when the proceeds equal the fees. This will handle sales
+  // made solely to cover annual account fees, where there is no money
+  // transferred.
+  if (rc) {
+    if (!d->m_parent->totalAmount().isZero()) {
+      rc &= haveAssetAccount();
+    }
+  }
   return rc;
 }
 
@@ -511,13 +527,14 @@ bool Sell::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, MyMoneySpl
       !createCategorySplits(t, interestAccountWidget, interestAmountWidget, MyMoneyMoney::MINUS_ONE, interestSplits, m_interestSplits))
     return false;
 
-  createAssetAccountSplit(assetAccountSplit, s0);
+  const auto total = sumSplits(s0, feeSplits, interestSplits);
+  if (!total.isZero()) {
+    createAssetAccountSplit(assetAccountSplit, s0);
+    assetAccountSplit.setValue(-total);
 
-  MyMoneyMoney total = sumSplits(s0, feeSplits, interestSplits);
-  assetAccountSplit.setValue(-total);
-
-  if (!d->m_parent->setupPrice(t, assetAccountSplit))
-    return false;
+    if (!d->m_parent->setupPrice(t, assetAccountSplit))
+      return false;
+  }
 
   return true;
 }
@@ -538,13 +555,14 @@ eMyMoney::Split::InvestmentTransactionType Div::type() const
 
 void Div::showWidgets() const
 {
-  static const QStringList visibleWidgetIds = QStringList() << "asset-account" << "interest-amount" << "total" << "interest-account" << "fee-account";
+  static const QStringList visibleWidgetIds = QStringList() << "asset-account" << "interest-amount" << "fee-amount" << "total" << "interest-account" << "fee-account";
   setWidgetVisibility(visibleWidgetIds, true);
   static const QStringList hiddenWidgetIds = QStringList() << "shares" << "price";
   setWidgetVisibility(hiddenWidgetIds, false);
 
   setLabelText("interest-amount-label", i18n("Interest"));
   setLabelText("interest-label", i18n("Interest"));
+  setLabelText("fee-amount-label", i18n("Fees"));
   setLabelText("fee-label", i18n("Fees"));
   setLabelText("asset-label", i18n("Account"));
   setLabelText("total-label", i18nc("Total value", "Total"));
@@ -628,12 +646,9 @@ void Reinvest::showWidgets() const
     shareEdit->setPrecision(MyMoneyMoney::denomToPrec(d->m_parent->security().smallestAccountFraction()));
   }
 
-  if (auto intAmount = dynamic_cast<KMyMoneyEdit*>(haveWidget("interest-amount"))) {
-    intAmount->hide();
-    setLabelText("interest-amount-label", QString());
-    intAmount->setValue(MyMoneyMoney());
-  }
-
+  setLabelText("interest-amount-label", i18n("Interest"));
+  setLabelText("interest-label", i18n("Interest"));
+  setLabelText("fee-amount-label", i18n("Fees"));
   setLabelText("fee-label", i18n("Fees"));
   setLabelText("interest-label", i18n("Interest"));
   setLabelText("shares-label", i18n("Shares"));
@@ -884,14 +899,6 @@ bool Invest::Split::createTransaction(MyMoneyTransaction& t, MyMoneySplit& s0, M
 
   auto sharesEdit = dynamic_cast<KMyMoneyEdit*>(haveWidget("shares"));
 
-  KMyMoneyCategory* cat;
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("interest-account"));
-  if (cat)
-    cat->parentWidget()->hide();
-  cat = dynamic_cast<KMyMoneyCategory*>(haveWidget("fee-account"));
-  if (cat)
-    cat->parentWidget()->hide();
-
   s0.setAction(eMyMoney::Split::InvestmentTransactionType::SplitShares);
   if (sharesEdit)
     s0.setShares(sharesEdit->value().abs());
