@@ -1,19 +1,20 @@
-/***************************************************************************
-                             ksettingshome.cpp
-                             --------------------
-    copyright            : (C) 2005 by Thomas Baumgart
-    email                : ipwizard@users.sourceforge.net
-                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+ * Copyright 2005-2018  Thomas Baumgart <tbaumgart@kde.org>
+ * Copyright 2017       Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "ksettingshome.h"
 
@@ -34,7 +35,7 @@
 
 #include "ui_ksettingshome.h"
 
-#include "kmymoney/kmymoneyglobalsettings.h"
+#include "kmymoneysettings.h"
 #include "kmymoney/kmymoneyutils.h"
 #include "icons/icons.h"
 
@@ -100,7 +101,7 @@ void KSettingsHome::slotLoadItems()
   if (d->m_noNeedToUpdateList)
     return;
 
-  QStringList list = KMyMoneyGlobalSettings::itemList();
+  QStringList list = KMyMoneySettings::listOfItems();
   QStringList::ConstIterator it;
   d->ui->m_homePageList->clear();
   QListWidgetItem *sel = 0;
