@@ -603,6 +603,10 @@ void StdTransactionEditor::slotReloadEditWidgets()
 
 void StdTransactionEditor::slotUpdatePayee(const QString& payeeId)
 {
+  // in case of an empty payee, there is nothing to do
+  if (payeeId.isEmpty())
+    return;
+
   Q_D(StdTransactionEditor);
   // we have a new payee assigned to this transaction.
   // in case there is no category assigned, no value entered and no
@@ -811,7 +815,7 @@ void StdTransactionEditor::autoFill(const QString& payeeId)
         if (editNr && !editNr->text().isEmpty()) {
           s.setNumber(editNr->text());
         } else if (!s.number().isEmpty()) {
-          s.setNumber(KMyMoneyUtils::nextCheckNumber(d->m_account));
+          s.setNumber(KMyMoneyUtils::nextFreeCheckNumber(d->m_account));
         }
 
         // if the memos should not be used with autofill or
@@ -1633,6 +1637,9 @@ bool StdTransactionEditor::createTransaction(MyMoneyTransaction& t, const MyMone
     if (!s1.accountId().isEmpty())
       t.addSplit(s1);
 
+    // check if we need to add/update a VAT assignment
+    MyMoneyFile::instance()->updateVAT(t);
+
   } else {
     foreach (const auto split, splits) {
       s1 = split;
