@@ -1,30 +1,39 @@
-/***************************************************************************
-                             transactionmatcher.h
-                             ----------
-    begin                : Tue Jul 08 2008
-    copyright            : (C) 2008 by Thomas Baumgart
-    email                : Thomas Baumgart <ipwizard@users.sourceforge.net>
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+ * Copyright 2008-2015  Thomas Baumgart <tbaumgart@kde.org>
+ * Copyright 2015       Christian Dávid <christian-david@web.de>
+ * Copyright 2017-2018  Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef TRANSACTIONMATCHER_H
 #define TRANSACTIONMATCHER_H
 
-#include "mymoneytransaction.h"
-#include "mymoneyaccount.h"
+#include <qglobal.h>
 
+class MyMoneySplit;
+class MyMoneyTransaction;
+class MyMoneyAccount;
+
+class TransactionMatcherPrivate;
 class TransactionMatcher
 {
 public:
-  TransactionMatcher(const MyMoneyAccount& acc);
+  Q_DISABLE_COPY(TransactionMatcher)
+
+  explicit TransactionMatcher(const MyMoneyAccount& acc);
+  ~TransactionMatcher();
 
   /**
    * This method matches the manual entered transaction @p tm with the imported
@@ -93,8 +102,8 @@ public:
   void accept(const MyMoneyTransaction& t, const MyMoneySplit& s);
 
 private:
-  MyMoneyAccount            m_account;
+  TransactionMatcherPrivate * const d_ptr;
+  Q_DECLARE_PRIVATE(TransactionMatcher)
 };
 
-
 #endif
