@@ -20,6 +20,7 @@
 
 #include <pwd.h>
 #include <unistd.h>
+#include <clocale>
 
 #include <QString>
 
@@ -37,3 +38,19 @@ uint platformTools::processId()
 {
   return getpid();
 }
+
+platformTools::currencySymbolPosition_t platformTools::currencySymbolPosition(bool negativeValues)
+{
+  platformTools::currencySymbolPosition_t  rc = platformTools::AfterQuantityMoneyWithSpace;
+  struct lconv* lc = std::localeconv();
+  if (lc) {
+    const char precedes = negativeValues ? lc->n_cs_precedes : lc->p_cs_precedes;
+    const char space = negativeValues ? lc->n_sep_by_space : lc->p_sep_by_space;
+    if (precedes == 1) {
+      rc = (space == 1) ? platformTools::BeforeQuantityMoneyWithSpace : platformTools::BeforeQuantityMoney;
+    } else {
+      rc = (space == 1) ? platformTools::AfterQuantityMoneyWithSpace : platformTools::AfterQuantityMoney;
+    }
+  }
+  return rc;
+}
