@@ -1,23 +1,20 @@
-/***************************************************************************
- *   Copyright 2010  Cristian Onet onet.cristian@gmail.com                 *
- *   Copyright 2017  Łukasz Wojniłowicz lukasz.wojnilowicz@gmail.com       *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
- ***************************************************************************/
+/*
+ * Copyright 2010-2014  Cristian Oneț <onet.cristian@gmail.com>
+ * Copyright 2017-2018  Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "accountsmodel.h"
 
@@ -30,7 +27,6 @@
 // KDE Includes
 
 #include <KLocalizedString>
-#include <KColorScheme>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -43,27 +39,42 @@
 #include "mymoneyaccount.h"
 #include "mymoneysecurity.h"
 #include "mymoneyprice.h"
-#include "kmymoneyglobalsettings.h"
+#include "kmymoneysettings.h"
 #include "icons.h"
 #include "modelenums.h"
 #include "mymoneyenums.h"
+#include "viewenums.h"
 
 using namespace Icons;
 using namespace eAccountsModel;
 using namespace eMyMoney;
 
-class AccountsModel::Private
+class AccountsModelPrivate
 {
+  Q_DECLARE_PUBLIC(AccountsModel)
+
 public:
   /**
     * The pimpl.
     */
-  Private() :
-      m_file(MyMoneyFile::instance()) {
+  AccountsModelPrivate(AccountsModel *qq) :
+    q_ptr(qq),
+    m_file(MyMoneyFile::instance())
+  {
     m_columns.append(Column::Account);
   }
 
-  ~Private() {
+  virtual ~AccountsModelPrivate()
+  {
+  }
+
+  void init()
+  {
+    Q_Q(AccountsModel);
+    QStringList headerLabels;
+    for (const auto& column : m_columns)
+      headerLabels.append(q->getHeaderName(column));
+    q->setHorizontalHeaderLabels(headerLabels);
   }
 
   void loadPreferredAccount(const MyMoneyAccount &acc, QStandardItem *fromNode /*accounts' regular node*/, const int row, QStandardItem *toNode /*accounts' favourite node*/)
@@ -72,8 +83,7 @@ public:
       return;
 
     auto favRow = toNode->rowCount();
-    auto favItem = itemFromAccountId(toNode, acc.id());
-    if (favItem)
+    if (auto favItem = itemFromAccountId(toNode, acc.id()))
       favRow = favItem->row();
 
     for (auto i = 0; i < fromNode->columnCount(); ++i) {
@@ -94,9 +104,7 @@ public:
     */
   void loadSubaccounts(QStandardItem *node, QStandardItem *favoriteAccountsItem, const QStringList& subaccounts)
   {
-    if (subaccounts.isEmpty())
-      return;
-    foreach (const auto subaccStr, subaccounts) {
+    for (const auto& subaccStr : subaccounts) {
       const auto subacc = m_file->account(subaccStr);
 
       auto item = new QStandardItem(subacc.name());                         // initialize first column of subaccount
@@ -254,13 +262,8 @@ public:
       cell = new QStandardItem;
       node->setChild(row, colTotalValue, cell);
     }
-    QColor color;
-    if (valInstitution.isNegative())
-      color = KMyMoneyGlobalSettings::schemeColor(SchemeColor::Negative);
-    else
-      color = KColorScheme(QPalette::Active).foreground(KColorScheme::NormalText).color();
-
-    cell->setData(QVariant(color),                                                   Qt::ForegroundRole);
+    const auto fontColor = KMyMoneySettings::schemeColor(valInstitution.isNegative() ? SchemeColor::Negative : SchemeColor::Positive);
+    cell->setData(QVariant(fontColor),                                               Qt::ForegroundRole);
     cell->setData(QVariant(itInstitution->data(Qt::FontRole).value<QFont>()),        Qt::FontRole);
     cell->setData(QVariant(Qt::AlignRight | Qt::AlignVCenter),                       Qt::TextAlignmentRole);
     cell->setData(MyMoneyUtils::formatMoney(valInstitution, m_file->baseCurrency()), Qt::DisplayRole);
@@ -324,9 +327,11 @@ public:
       if (colNum != -1) {
         const auto accountValueStr = QVariant::fromValue(MyMoneyUtils::formatMoney(accountValue, m_file->baseCurrency()));
         getCell(colNum);
-        cell->setData(accountValueStr, Qt::DisplayRole);
-        cell->setData(font,       Qt::FontRole);
-        cell->setData(alignment,  Qt::TextAlignmentRole);
+        const auto fontColor = KMyMoneySettings::schemeColor(accountValue.isNegative() ? SchemeColor::Negative : SchemeColor::Positive);
+        cell->setData(QVariant(fontColor),  Qt::ForegroundRole);
+        cell->setData(accountValueStr,      Qt::DisplayRole);
+        cell->setData(font,                 Qt::FontRole);
+        cell->setData(alignment,            Qt::TextAlignmentRole);
       }
     }
 
@@ -336,15 +341,10 @@ public:
       if (colNum != -1) {
         const auto accountTotalValueStr = QVariant::fromValue(MyMoneyUtils::formatMoney(accountTotalValue, m_file->baseCurrency()));
         getCell(colNum);
-        QColor color;
-        if (accountTotalValue.isNegative())
-          color = KMyMoneyGlobalSettings::schemeColor(SchemeColor::Negative);
-        else
-          color = KColorScheme(QPalette::Active).foreground(KColorScheme::NormalText).color();
-
+        const auto fontColor = KMyMoneySettings::schemeColor(accountTotalValue.isNegative() ? SchemeColor::Negative : SchemeColor::Positive);
         cell->setData(accountTotalValueStr, Qt::DisplayRole);
         cell->setData(font,                 Qt::FontRole);
-        cell->setData(QVariant(color),      Qt::ForegroundRole);
+        cell->setData(QVariant(fontColor),  Qt::ForegroundRole);
         cell->setData(alignment,            Qt::TextAlignmentRole);
       }
     }
@@ -418,13 +418,13 @@ public:
     MyMoneyMoney value = balance;
     {
       QList<MyMoneyPrice>::const_iterator it_p;
-      QString security = account.currencyId();
+      QString securityID = account.currencyId();
       for (it_p = prices.constBegin(); it_p != prices.constEnd(); ++it_p) {
-        value = (value * (MyMoneyMoney::ONE / (*it_p).rate(security))).convertPrecision(m_file->security(security).pricePrecision());
-        if ((*it_p).from() == security)
-          security = (*it_p).to();
+        value = (value * (MyMoneyMoney::ONE / (*it_p).rate(securityID))).convertPrecision(m_file->security(securityID).pricePrecision());
+        if ((*it_p).from() == securityID)
+          securityID = (*it_p).to();
         else
-          security = (*it_p).from();
+          securityID = (*it_p).from();
       }
       value = value.convert(m_file->baseCurrency().smallestAccountFraction());
     }
@@ -495,7 +495,7 @@ public:
   QStandardItem *itemFromAccountId(QStandardItemModel *model, const QString &accountId)
   {
     const auto list = model->match(model->index(0, 0), (int)Role::ID, QVariant(accountId), -1, Qt::MatchFlags(Qt::MatchExactly | Qt::MatchCaseSensitive | Qt::MatchRecursive));
-    foreach (const QModelIndex &index, list) {
+    for (const auto& index : list) {
       // always return the account which is not the child of the favorite accounts item
       if (index.parent().data((int)Role::ID).toString() != AccountsModel::favoritesAccountId)
         return model->itemFromIndex(index);
@@ -503,6 +503,8 @@ public:
     return nullptr;
   }
 
+  AccountsModel *q_ptr;
+
   /**
     * Used to load the accounts data.
     */
@@ -525,39 +527,36 @@ public:
   static const QString m_accountsModelColumnSelection;
 };
 
-const QString AccountsModel::Private::m_accountsModelConfGroup = QStringLiteral("AccountsModel");
-const QString AccountsModel::Private::m_accountsModelColumnSelection = QStringLiteral("ColumnSelection");
+const QString AccountsModelPrivate::m_accountsModelConfGroup = QStringLiteral("AccountsModel");
+const QString AccountsModelPrivate::m_accountsModelColumnSelection = QStringLiteral("ColumnSelection");
 
 const QString AccountsModel::favoritesAccountId(QStringLiteral("Favorites"));
 
 /**
   * The constructor is private so that only the @ref Models object can create such an object.
   */
-AccountsModel::AccountsModel(QObject *parent /*= 0*/)
-    : QStandardItemModel(parent), d(new Private)
+AccountsModel::AccountsModel(QObject *parent) :
+  QStandardItemModel(parent),
+  d_ptr(new AccountsModelPrivate(this))
 {
-  init();
+  Q_D(AccountsModel);
+  d->init();
 }
 
-AccountsModel::AccountsModel(Private* const priv, QObject *parent /*= 0*/)
-    : QStandardItemModel(parent), d(priv)
+AccountsModel::AccountsModel(AccountsModelPrivate &dd, QObject *parent) :
+  QStandardItemModel(parent),
+  d_ptr(&dd)
 {
-  init();
+  Q_D(AccountsModel);
+  d->init();
 }
 
 AccountsModel::~AccountsModel()
 {
+  Q_D(AccountsModel);
   delete d;
 }
 
-void AccountsModel::init()
-{
-  QStringList headerLabels;
-  foreach (const auto column, d->m_columns)
-    headerLabels.append(getHeaderName(column));
-  setHorizontalHeaderLabels(headerLabels);
-}
-
 /**
   * Perform the initial load of the model data
   * from the @ref MyMoneyFile.
@@ -565,7 +564,8 @@ void AccountsModel::init()
   */
 void AccountsModel::load()
 {
-  this->blockSignals(true);
+  Q_D(AccountsModel);
+  blockSignals(true);
   QStandardItem *rootItem = invisibleRootItem();
 
   QFont font;
@@ -586,13 +586,13 @@ void AccountsModel::load()
   }
 
   // adding account categories (asset, liability, etc.) node
-  QVector <Account::Type> categories {
+  const QVector <Account::Type> categories {
     Account::Type::Asset, Account::Type::Liability,
     Account::Type::Income, Account::Type::Expense,
     Account::Type::Equity
   };
 
-  foreach (const auto category, categories) {
+  for (const auto category : categories) {
     MyMoneyAccount account;
     QString accountName;
     int displayOrder;
@@ -646,7 +646,7 @@ void AccountsModel::load()
     }
 
     // adding accounts (specific bank/investment accounts) belonging to given accounts category
-    foreach (const auto accStr, account.accountList()) {
+    for (const auto& accStr : account.accountList()) {
       const auto acc = d->m_file->account(accStr);
 
       auto item = new QStandardItem(acc.name());
@@ -656,7 +656,7 @@ void AccountsModel::load()
       // filter out stocks with zero balance if requested by user
       for (auto subaccStr = subaccountsStr.begin(); subaccStr != subaccountsStr.end();) {
         const auto subacc = d->m_file->account(*subaccStr);
-        if (subacc.isInvest() && KMyMoneyGlobalSettings::hideZeroBalanceEquities() && subacc.balance().isZero())
+        if (subacc.isInvest() && KMyMoneySettings::hideZeroBalanceEquities() && subacc.balance().isZero())
           subaccStr = subaccountsStr.erase(subaccStr);
         else
           ++subaccStr;
@@ -672,9 +672,9 @@ void AccountsModel::load()
     d->setAccountData(rootItem, accountsItem->row(), account, d->m_columns);
   }
 
+  blockSignals(false);
   checkNetWorth();
   checkProfit();
-  this->blockSignals(false);
 }
 
 QModelIndex AccountsModel::accountById(const QString& id) const
@@ -693,11 +693,13 @@ QModelIndex AccountsModel::accountById(const QString& id) const
 
 QList<Column> *AccountsModel::getColumns()
 {
+  Q_D(AccountsModel);
   return &d->m_columns;
 }
 
 void AccountsModel::setColumnVisibility(const Column column, const bool show)
 {
+  Q_D(AccountsModel);
   const auto ixCol = d->m_columns.indexOf(column);  // get column index in our column's map
   if (!show && ixCol != -1) {                       // start removing column row by row from bottom to up
     d->m_columns.removeOne(column);                 // remove it from our column's map
@@ -743,7 +745,7 @@ void AccountsModel::setColumnVisibility(const Column column, const bool show)
           childItem->insertColumns(newColPos, 1);
           if (childItem->hasChildren())
             self(self, childItem);
-          this->d->setAccountData(item, j, childItem->data((int)Role::Account).value<MyMoneyAccount>(), QList<Column> {column});
+          d->setAccountData(item, j, childItem->data((int)Role::Account).value<MyMoneyAccount>(), QList<Column> {column});
         }
         return true;
       };
@@ -795,6 +797,7 @@ QString AccountsModel::getHeaderName(const Column column)
   */
 void AccountsModel::checkNetWorth()
 {
+  Q_D(AccountsModel);
   // compute the net woth
   QModelIndexList assetList = match(index(0, 0),
                                     (int)Role::ID,
@@ -818,7 +821,7 @@ void AccountsModel::checkNetWorth()
   }
   if (d->m_lastNetWorth != netWorth) {
     d->m_lastNetWorth = netWorth;
-    emit netWorthChanged(d->m_lastNetWorth);
+    emit netWorthChanged(QVariantList {QVariant::fromValue(d->m_lastNetWorth)}, eView::Intent::UpdateNetWorth);
   }
 }
 
@@ -827,6 +830,7 @@ void AccountsModel::checkNetWorth()
   */
 void AccountsModel::checkProfit()
 {
+  Q_D(AccountsModel);
   // compute the profit
   const auto incomeList = match(index(0, 0),
                                 (int)Role::ID,
@@ -850,12 +854,13 @@ void AccountsModel::checkProfit()
   }
   if (d->m_lastProfit != profit) {
     d->m_lastProfit = profit;
-    emit profitChanged(d->m_lastProfit);
+    emit profitChanged(QVariantList {QVariant::fromValue(d->m_lastProfit)}, eView::Intent::UpdateProfit);
   }
 }
 
 MyMoneyMoney AccountsModel::accountValue(const MyMoneyAccount &account, const MyMoneyMoney &balance)
 {
+  Q_D(AccountsModel);
   return d->value(account, balance);
 }
 
@@ -864,19 +869,20 @@ MyMoneyMoney AccountsModel::accountValue(const MyMoneyAccount &account, const My
   */
 void AccountsModel::slotReconcileAccount(const MyMoneyAccount &account, const QDate &reconciliationDate, const MyMoneyMoney &endingBalance)
 {
+  Q_D(AccountsModel);
   Q_UNUSED(reconciliationDate)
   Q_UNUSED(endingBalance)
   if (d->m_reconciledAccount.id() != account.id()) {
     // first clear the flag of the old reconciliation account
     if (!d->m_reconciledAccount.id().isEmpty()) {
       const auto list = match(index(0, 0), (int)Role::ID, QVariant(d->m_reconciledAccount.id()), -1, Qt::MatchFlags(Qt::MatchExactly | Qt::MatchCaseSensitive | Qt::MatchRecursive));
-      foreach (const auto index, list)
+      for (const auto& index : list)
         setData(index, QVariant(QIcon(account.accountPixmap(false))), Qt::DecorationRole);
     }
 
     // then set the reconciliation flag of the new reconciliation account
     const auto list = match(index(0, 0), (int)Role::ID, QVariant(account.id()), -1, Qt::MatchFlags(Qt::MatchExactly | Qt::MatchCaseSensitive | Qt::MatchRecursive));
-    foreach (const auto index, list)
+    for (const auto& index : list)
       setData(index, QVariant(QIcon(account.accountPixmap(true))), Qt::DecorationRole);
     d->m_reconciledAccount = account;
   }
@@ -886,30 +892,29 @@ void AccountsModel::slotReconcileAccount(const MyMoneyAccount &account, const QD
   * Notify the model that an object has been added. An action is performed only if the object is an account.
   *
   */
-void AccountsModel::slotObjectAdded(File::Object objType, const MyMoneyObject * const obj)
+void AccountsModel::slotObjectAdded(File::Object objType, const QString& id)
 {
+  Q_D(AccountsModel);
   if (objType != File::Object::Account)
     return;
 
-  const MyMoneyAccount * const account = dynamic_cast<const MyMoneyAccount * const>(obj);
-  if (!account)
-    return;
+  const auto account = MyMoneyFile::instance()->account(id);
 
   auto favoriteAccountsItem = d->itemFromAccountId(this, favoritesAccountId);
-  auto parentAccountItem = d->itemFromAccountId(this, account->parentAccountId());
-  auto item = d->itemFromAccountId(parentAccountItem, account->id());
+  auto parentAccountItem = d->itemFromAccountId(this, account.parentAccountId());
+  auto item = d->itemFromAccountId(parentAccountItem, account.id());
   if (!item) {
-    item = new QStandardItem(account->name());
+    item = new QStandardItem(account.name());
     parentAccountItem->appendRow(item);
     item->setEditable(false);
   }
   // load the sub-accounts if there are any - there could be sub accounts if this is an add operation
   // that was triggered in slotObjectModified on an already existing account which went trough a hierarchy change
-  d->loadSubaccounts(item, favoriteAccountsItem, account->accountList());
+  d->loadSubaccounts(item, favoriteAccountsItem, account.accountList());
 
   const auto row = item->row();
-  d->setAccountData(parentAccountItem, row, *account, d->m_columns);
-  d->loadPreferredAccount(*account, parentAccountItem, row, favoriteAccountsItem);
+  d->setAccountData(parentAccountItem, row, account, d->m_columns);
+  d->loadPreferredAccount(account, parentAccountItem, row, favoriteAccountsItem);
 
   checkNetWorth();
   checkProfit();
@@ -919,34 +924,38 @@ void AccountsModel::slotObjectAdded(File::Object objType, const MyMoneyObject *
   * Notify the model that an object has been modified. An action is performed only if the object is an account.
   *
   */
-void AccountsModel::slotObjectModified(File::Object objType, const MyMoneyObject * const obj)
+void AccountsModel::slotObjectModified(File::Object objType, const QString& id)
 {
+  Q_D(AccountsModel);
   if (objType != File::Object::Account)
     return;
 
-  const MyMoneyAccount * const account = dynamic_cast<const MyMoneyAccount * const>(obj);
-  if (!account)
+  const auto account = MyMoneyFile::instance()->account(id);
+  auto accountItem = d->itemFromAccountId(this, id);
+  if (!accountItem) {
+    qDebug() << "Unexpected null accountItem in AccountsModel::slotObjectModified";
     return;
-  auto favoriteAccountsItem = d->itemFromAccountId(this, favoritesAccountId);
-  auto accountItem = d->itemFromAccountId(this, account->id());
+  }
+
   const auto oldAccount = accountItem->data((int)Role::Account).value<MyMoneyAccount>();
-  if (oldAccount.parentAccountId() == account->parentAccountId()) {
+  if (oldAccount.parentAccountId() == account.parentAccountId()) {
     // the hierarchy did not change so update the account data
     auto parentAccountItem = accountItem->parent();
     if (!parentAccountItem)
       parentAccountItem = this->invisibleRootItem();
     const auto row = accountItem->row();
-    d->setAccountData(parentAccountItem, row, *account, d->m_columns);
+    d->setAccountData(parentAccountItem, row, account, d->m_columns);
     // and the child of the favorite item if the account is a favorite account or it's favorite status has just changed
-    auto favItem = d->itemFromAccountId(favoriteAccountsItem, account->id());
-    if (account->value("PreferredAccount") == QLatin1String("Yes"))
-      d->loadPreferredAccount(*account, parentAccountItem, row, favoriteAccountsItem);
-    else if (favItem)
-      favoriteAccountsItem->removeRow(favItem->row()); // it's not favorite anymore
+    if (auto favoriteAccountsItem = d->itemFromAccountId(this, favoritesAccountId)) {
+      if (account.value("PreferredAccount") == QLatin1String("Yes"))
+        d->loadPreferredAccount(account, parentAccountItem, row, favoriteAccountsItem);
+      else if (auto favItem = d->itemFromAccountId(favoriteAccountsItem, account.id()))
+        favoriteAccountsItem->removeRow(favItem->row()); // it's not favorite anymore
+    }
   } else {
     // this means that the hierarchy was changed - simulate this with a remove followed by and add operation
     slotObjectRemoved(File::Object::Account, oldAccount.id());
-    slotObjectAdded(File::Object::Account, obj);
+    slotObjectAdded(File::Object::Account, id);
   }
 
   checkNetWorth();
@@ -962,8 +971,8 @@ void AccountsModel::slotObjectRemoved(File::Object objType, const QString& id)
   if (objType != File::Object::Account)
     return;
 
-  auto list = match(index(0, 0), (int)Role::ID, id, -1, Qt::MatchFlags(Qt::MatchExactly | Qt::MatchRecursive));
-  foreach (const auto index, list)
+  const auto list = match(index(0, 0), (int)Role::ID, id, -1, Qt::MatchFlags(Qt::MatchExactly | Qt::MatchRecursive));
+  for (const auto& index : list)
     removeRow(index.row(), index.parent());
 
   checkNetWorth();
@@ -975,6 +984,7 @@ void AccountsModel::slotObjectRemoved(File::Object objType, const QString& id)
   */
 void AccountsModel::slotBalanceOrValueChanged(const MyMoneyAccount &account)
 {
+  Q_D(AccountsModel);
   auto itParent = d->itemFromAccountId(this, account.id()); // get node of account in model
   auto isTopLevel = false;                                  // it could be top-level but we don't know it yet
   while (itParent && !isTopLevel) {                         // loop in which we set total values and balances from the bottom to the top
@@ -998,9 +1008,18 @@ void AccountsModel::slotBalanceOrValueChanged(const MyMoneyAccount &account)
 /**
   * The pimpl of the @ref InstitutionsModel derived from the pimpl of the @ref AccountsModel.
   */
-class InstitutionsModel::InstitutionsPrivate : public AccountsModel::Private
+class InstitutionsModelPrivate : public AccountsModelPrivate
 {
 public:
+  InstitutionsModelPrivate(InstitutionsModel *qq) :
+    AccountsModelPrivate(qq)
+  {
+  }
+
+  ~InstitutionsModelPrivate() override
+  {
+  }
+
   /**
     * Function to get the institution item from an institution id.
     *
@@ -1033,6 +1052,7 @@ public:
       const auto investmentAccount = m_file->account(account.parentAccountId());  // ...get investment account it's under and...
       idInstitution = investmentAccount.institutionId();                          // ...get institution from investment account
     }
+
     auto itInstitution = institutionItemFromId(model, idInstitution);
     auto itAccount = itemFromAccountId(itInstitution, account.id());  // check if account already exists under institution
     // only stock accounts are added to their parent in the institutions view
@@ -1083,8 +1103,12 @@ public:
   * The institution model contains the accounts grouped by institution.
   *
   */
-InstitutionsModel::InstitutionsModel(QObject *parent /*= 0*/)
-    : AccountsModel(new InstitutionsPrivate, parent)
+InstitutionsModel::InstitutionsModel(QObject *parent) :
+  AccountsModel(*new InstitutionsModelPrivate(this), parent)
+{
+}
+
+InstitutionsModel::~InstitutionsModel()
 {
 }
 
@@ -1095,29 +1119,28 @@ InstitutionsModel::InstitutionsModel(QObject *parent /*= 0*/)
   */
 void InstitutionsModel::load()
 {
+  Q_D(InstitutionsModel);
   // create items for all the institutions
-  QList<MyMoneyInstitution> institutionList;
-  d->m_file->institutionList(institutionList);
+  auto institutionList = d->m_file->institutionList();
   MyMoneyInstitution none;
   none.setName(i18n("Accounts with no institution assigned"));
   institutionList.append(none);
-  auto modelUtils = static_cast<InstitutionsPrivate *>(d);
-  foreach (const auto institution, institutionList)   // add all known institutions as top-level nodes
-    modelUtils->addInstitutionItem(this, institution);
+  for (const auto& institution : institutionList)   // add all known institutions as top-level nodes
+    d->addInstitutionItem(this, institution);
 
   QList<MyMoneyAccount> accountsList;
   QList<MyMoneyAccount> stocksList;
   d->m_file->accountList(accountsList);
-  foreach (const auto account, accountsList) {  // add account nodes under institution nodes...
+  for (const auto& account : accountsList) {  // add account nodes under institution nodes...
     if (account.isInvest())                     // ...but wait with stocks until investment accounts appear
       stocksList.append(account);
     else
-      modelUtils->loadInstitution(this, account);
+      d->loadInstitution(this, account);
   }
 
-  foreach (const auto stock, stocksList) {
-    if (!(KMyMoneyGlobalSettings::hideZeroBalanceEquities() && stock.balance().isZero()))
-      modelUtils->loadInstitution(this, stock);
+  for (const auto& stock : stocksList) {
+    if (!(KMyMoneySettings::hideZeroBalanceEquities() && stock.balance().isZero()))
+      d->loadInstitution(this, stock);
   }
 
   for (auto i = 0 ; i < rowCount(); ++i)
@@ -1128,38 +1151,36 @@ void InstitutionsModel::load()
   * Notify the model that an object has been added. An action is performed only if the object is an account or an institution.
   *
   */
-void InstitutionsModel::slotObjectAdded(File::Object objType, const MyMoneyObject * const obj)
+void InstitutionsModel::slotObjectAdded(File::Object objType, const QString& id)
 {
-  auto modelUtils = static_cast<InstitutionsPrivate *>(d);
+  Q_D(InstitutionsModel);
   if (objType == File::Object::Institution) {
     // if an institution was added then add the item which will represent it
-    const MyMoneyInstitution * const institution = dynamic_cast<const MyMoneyInstitution * const>(obj);
-    if (!institution)
-      return;
-    modelUtils->addInstitutionItem(this, *institution);
+    const auto institution = MyMoneyFile::instance()->institution(id);
+    d->addInstitutionItem(this, institution);
   }
 
   if (objType != File::Object::Account)
     return;
 
   // if an account was added then add the item which will represent it only for real accounts
-  const MyMoneyAccount * const account = dynamic_cast<const MyMoneyAccount * const>(obj);
+  const auto account = MyMoneyFile::instance()->account(id);
   // nothing to do for root accounts and categories
-  if (!account || account->parentAccountId().isEmpty() || account->isIncomeExpense())
+  if (account.parentAccountId().isEmpty() || account.isIncomeExpense())
     return;
 
   // load the account into the institution
-  modelUtils->loadInstitution(this, *account);
+  d->loadInstitution(this, account);
 
   // load the investment sub-accounts if there are any - there could be sub-accounts if this is an add operation
   // that was triggered in slotObjectModified on an already existing account which went trough a hierarchy change
-  const auto sAccounts = account->accountList();
+  const auto sAccounts = account.accountList();
   if (!sAccounts.isEmpty()) {
     QList<MyMoneyAccount> subAccounts;
     d->m_file->accountList(subAccounts, sAccounts);
-    foreach (const auto subAccount, subAccounts) {
+    for (const auto& subAccount : subAccounts) {
       if (subAccount.isInvest()) {
-        modelUtils->loadInstitution(this, subAccount);
+        d->loadInstitution(this, subAccount);
       }
     }
   }
@@ -1169,37 +1190,37 @@ void InstitutionsModel::slotObjectAdded(File::Object objType, const MyMoneyObjec
   * Notify the model that an object has been modified. An action is performed only if the object is an account or an institution.
   *
   */
-void InstitutionsModel::slotObjectModified(File::Object objType, const MyMoneyObject * const obj)
+void InstitutionsModel::slotObjectModified(File::Object objType, const QString& id)
 {
+  Q_D(InstitutionsModel);
   if (objType == File::Object::Institution) {
     // if an institution was modified then modify the item which represents it
-    const MyMoneyInstitution * const institution = dynamic_cast<const MyMoneyInstitution * const>(obj);
-    if (!institution)
-      return;
-    auto institutionItem = static_cast<InstitutionsPrivate *>(d)->institutionItemFromId(this, institution->id());
-    institutionItem->setData(institution->name(), Qt::DisplayRole);
-    institutionItem->setData(QVariant::fromValue(*institution), (int)Role::Account);
-    institutionItem->setIcon(institution->pixmap());
+    const auto institution = MyMoneyFile::instance()->institution(id);
+    if (auto institutionItem = d->institutionItemFromId(this, id)) {
+      institutionItem->setData(institution.name(), Qt::DisplayRole);
+      institutionItem->setData(QVariant::fromValue(institution), (int)Role::Account);
+      institutionItem->setIcon(MyMoneyInstitution::pixmap());
+    }
   }
 
   if (objType != File::Object::Account)
     return;
 
   // if an account was modified then modify the item which represents it
-  const MyMoneyAccount * const account = dynamic_cast<const MyMoneyAccount * const>(obj);
+  const auto account = MyMoneyFile::instance()->account(id);
   // nothing to do for root accounts, categories and equity accounts since they don't have a representation in this model
-  if (!account || account->parentAccountId().isEmpty() || account->isIncomeExpense() || account->accountType() == Account::Type::Equity)
+  if (account.parentAccountId().isEmpty() || account.isIncomeExpense() || account.accountType() == Account::Type::Equity)
     return;
 
-  auto accountItem = d->itemFromAccountId(this, account->id());
+  auto accountItem = d->itemFromAccountId(this, account.id());
   const auto oldAccount = accountItem->data((int)Role::Account).value<MyMoneyAccount>();
-  if (oldAccount.institutionId() == account->institutionId()) {
+  if (oldAccount.institutionId() == account.institutionId()) {
     // the hierarchy did not change so update the account data
-    d->setAccountData(accountItem->parent(), accountItem->row(), *account, d->m_columns);
+    d->setAccountData(accountItem->parent(), accountItem->row(), account, d->m_columns);
   } else {
     // this means that the hierarchy was changed - simulate this with a remove followed by and add operation
     slotObjectRemoved(File::Object::Account, oldAccount.id());
-    slotObjectAdded(File::Object::Account, obj);
+    slotObjectAdded(File::Object::Account, id);
   }
 }
 
@@ -1209,10 +1230,10 @@ void InstitutionsModel::slotObjectModified(File::Object objType, const MyMoneyOb
   */
 void InstitutionsModel::slotObjectRemoved(File::Object objType, const QString& id)
 {
+  Q_D(InstitutionsModel);
   if (objType == File::Object::Institution) {
     // if an institution was removed then remove the item which represents it
-    auto itInstitution = static_cast<InstitutionsPrivate *>(d)->institutionItemFromId(this, id);
-    if (itInstitution)
+    if (auto itInstitution = d->institutionItemFromId(this, id))
       removeRow(itInstitution->row(), itInstitution->index().parent());
   }
 
@@ -1225,8 +1246,8 @@ void InstitutionsModel::slotObjectRemoved(File::Object objType, const QString& i
     return; // this could happen if the account isIncomeExpense
 
   const auto account = itAccount->data((int)Role::Account).value<MyMoneyAccount>();
-  auto itInstitution = d->itemFromAccountId(this, account.institutionId());
-
-  AccountsModel::slotObjectRemoved(objType, id);
-  d->setInstitutionTotalValue(invisibleRootItem(), itInstitution->row());
+  if (auto itInstitution = d->itemFromAccountId(this, account.institutionId())) {
+    AccountsModel::slotObjectRemoved(objType, id);
+    d->setInstitutionTotalValue(invisibleRootItem(), itInstitution->row());
+  }
 }
