@@ -1,27 +1,26 @@
-/***************************************************************************
- *   Copyright 2010  Cristian Onet onet.cristian@gmail.com                 *
- *   Copyright 2017  Łukasz Wojniłowicz lukasz.wojnilowicz@gmail.com       *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
- ***************************************************************************/
+/*
+ * Copyright 2010-2014  Cristian Oneț <onet.cristian@gmail.com>
+ * Copyright 2017-2018  Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef ACCOUNTSPROXYMODEL_H
 #define ACCOUNTSPROXYMODEL_H
 
+#include "kmm_models_export.h"
+
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -58,13 +57,15 @@
 namespace eMyMoney { namespace Account {enum class Type;} }
 namespace eAccountsModel { enum class Column; }
 
-class AccountsProxyModel : public KRecursiveFilterProxyModel
+class AccountsProxyModelPrivate;
+class KMM_MODELS_EXPORT AccountsProxyModel : public KRecursiveFilterProxyModel
 {
   Q_OBJECT
+  Q_DISABLE_COPY(AccountsProxyModel)
 
 public:
   explicit AccountsProxyModel(QObject *parent = nullptr);
-  ~AccountsProxyModel();
+  virtual ~AccountsProxyModel();
 
   void addAccountType(eMyMoney::Account::Type type);
   void addAccountGroup(const QVector<eMyMoney::Account::Type> &groups);
@@ -85,9 +86,10 @@ public:
 
   void setSourceColumns(QList<eAccountsModel::Column> *columns);
 
-  QList<eAccountsModel::Column> *m_mdlColumns;
-
 protected:
+  const QScopedPointer<AccountsProxyModelPrivate> d_ptr;
+  AccountsProxyModel(AccountsProxyModelPrivate &dd, QObject *parent);
+
   bool lessThan(const QModelIndex &left, const QModelIndex &right) const override;
   bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const override;
   bool acceptSourceItem(const QModelIndex &source) const;
@@ -100,8 +102,7 @@ Q_SIGNALS:
   void unusedIncomeExpenseAccountHidden() const;
 
 private:
-  class Private;
-  Private* const d;
+  Q_DECLARE_PRIVATE(AccountsProxyModel)
 };
 
 #endif
