@@ -1,26 +1,21 @@
-/***************************************************************************
-                          mymoneystatement.cpp
-                          -------------------
-    begin                : Mon Aug 30 2004
-    copyright            : (C) 2000-2004 by Michael Edwardes
-    email                : mte@users.sourceforge.net
-                           Javier Campos Morales <javi_c@users.sourceforge.net>
-                           Felix Rodriguez <frodriguez@users.sourceforge.net>
-                           John C <thetacoturtle@users.sourceforge.net>
-                           Thomas Baumgart <ipwizard@users.sourceforge.net>
-                           Kevin Tambascio <ktambascio@users.sourceforge.net>
-                           Ace Jones <acejones@users.sourceforge.net>
-                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+ * Copyright 2004-2006  Ace Jones <acejones@users.sourceforge.net>
+ * Copyright 2005-2018  Thomas Baumgart <tbaumgart@kde.org>
+ * Copyright 2017-2018  Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "mymoneystatement.h"
 
@@ -258,7 +253,7 @@ bool MyMoneyStatement::read(const QDomElement& _e)
         t.m_strBankID = c.attribute(getAttrName(Statement::Attribute::BankID));
         t.m_reconcile = static_cast<eMyMoney::Split::State>(c.attribute(getAttrName(Statement::Attribute::Reconcile)).toInt());
 
-        auto txt = c.attribute(getAttrName(Statement::Attribute::Action), txAction[eMyMoney::Transaction::Action::Buy]);
+        txt = c.attribute(getAttrName(Statement::Attribute::Action), txAction[eMyMoney::Transaction::Action::Buy]);
         t.m_eAction = txAction.key(txt, t.m_eAction);
 
         if (m_eType == eMyMoney::Statement::Type::Investment) {
@@ -268,9 +263,9 @@ bool MyMoneyStatement::read(const QDomElement& _e)
         }
 
         // process splits (if any)
-        QDomNode child = c.firstChild();
+        child = c.firstChild();
         while (!child.isNull() && child.isElement()) {
-          QDomElement c = child.toElement();
+          c = child.toElement();
           if (c.tagName() == getElName(Statement::Element::Split)) {
             MyMoneyStatement::Split s;
             s.m_accountId = c.attribute(getAttrName(Statement::Attribute::AccountID));
@@ -382,3 +377,17 @@ bool MyMoneyStatement::readXMLFile(MyMoneyStatement& _s, const QString& _filenam
 
   return result;
 }
+
+QDate MyMoneyStatement::statementEndDate() const
+{
+  if (m_dateEnd.isValid())
+    return m_dateEnd;
+
+  QDate postDate;
+  for(auto t : m_listTransactions) {
+    if (t.m_datePosted > postDate) {
+      postDate = t.m_datePosted;
+    }
+  }
+  return postDate;
+}
