@@ -92,6 +92,10 @@ KOnlineBankingSetupWizard::KOnlineBankingSetupWizard(QWidget *parent):
   //set password field according to KDE preferences
   m_editPassword->setPasswordMode(true);
 
+  // make sure to not exceed data fields
+  m_editUsername->setMaxLength(OFX_USERID_LENGTH-1);
+  m_editPassword->setMaxLength(OFX_USERPASS_LENGTH-1);
+
   KListWidgetSearchLine* searchLine = new KListWidgetSearchLine(autoTab, m_listFi);
   vboxLayout1->insertWidget(0, searchLine);
   QTimer::singleShot(20, searchLine, SLOT(setFocus()));
@@ -328,9 +332,11 @@ bool KOnlineBankingSetupWizard::finishLoginPage()
   while (m_it_info != m_bankInfo.constEnd()) {
     OfxFiLogin fi;
     memset(&fi, 0, sizeof(OfxFiLogin));
-    strncpy(fi.fid, (*m_it_info).fid, OFX_FID_LENGTH - 1);
-    strncpy(fi.org, (*m_it_info).org, OFX_ORG_LENGTH - 1);
-    strncpy(fi.userid, username.toLatin1(), OFX_USERID_LENGTH - 1);
+    Q_ASSERT(sizeof(fi.fid) == sizeof((*m_it_info).fid));
+    Q_ASSERT(sizeof(fi.org) == sizeof((*m_it_info).org));
+    memcpy(fi.fid, (*m_it_info).fid, OFX_FID_LENGTH - 1);
+    memcpy(fi.org, (*m_it_info).org, OFX_ORG_LENGTH - 1);
+    strncpy(fi.userid, username.toLatin1().left(OFX_USERID_LENGTH - 1), OFX_USERID_LENGTH - 1);
     strncpy(fi.userpass, password.toLatin1(), OFX_USERPASS_LENGTH - 1);
 #ifdef LIBOFX_HAVE_CLIENTUID
     strncpy(fi.clientuid, clientUid.toLatin1(), OFX_CLIENTUID_LENGTH - 1);
@@ -356,7 +362,7 @@ bool KOnlineBankingSetupWizard::finishLoginPage()
     QString hver = m_headerVersion->headerVersion();
     strncpy(fi.header_version, hver.toLatin1(), OFX_HEADERVERSION_LENGTH - 1);
 
-    QUrl filename(QString("file://%1response.ofx").arg(QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QLatin1Char('/') + ""));
+    QUrl filename(QString("file://%1response.ofx").arg(QStandardPaths::writableLocation(QStandardPaths::DataLocation) + QLatin1Char('/')));
     QByteArray req(libofx_request_accountinfo(&fi));
     // because the event loop is running while the request is performed disable the back button
     // (this function is not reentrant so the application might crash when back/next are used)
@@ -372,7 +378,7 @@ bool KOnlineBankingSetupWizard::finishLoginPage()
     ofx_set_account_cb(ctx, ofxAccountCallback, this);
     ofx_set_status_cb(ctx, ofxStatusCallback, this);
     // Add resulting accounts to the account list
-    libofx_proc_file(ctx, filename.path().toLatin1(), AUTODETECT);
+    libofx_proc_file(ctx, filename.toLocalFile().toLatin1(), AUTODETECT);
     libofx_free_context(ctx);
 
     ++m_it_info;
