@@ -1,19 +1,20 @@
-/***************************************************************************
-                          ofximporter.cpp
-                             -------------------
-    begin                : Sat Jan 01 2005
-    copyright            : (C) 2005 by Ace Jones
-    email                : Ace Jones <acejones@users.sourceforge.net>
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+ * Copyright 2005       Ace Jones acejones@users.sourceforge.net
+ * Copyright 2010-2018  Thomas Baumgart tbaumgart@kde.org
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "ofximporter.h"
 
@@ -46,10 +47,12 @@
 #include "mymoneyaccount.h"
 #include "mymoneyexception.h"
 #include "mymoneystatement.h"
+#include "mymoneystatementreader.h"
 #include "statementinterface.h"
 #include "importinterface.h"
+#include "viewinterface.h"
 #include "ui_importoption.h"
-#include "kmymoneyglobalsettings.h"
+#include "kmymoneyutils.h"
 
 //#define DEBUG_LIBOFX
 
@@ -59,7 +62,7 @@ class OFXImporter::Private
 {
 public:
   Private() : m_valid(false), m_preferName(PreferId), m_walletIsOpen(false), m_statusDlg(0), m_wallet(0),
-              m_updateStartDate(QDate(1900,1,1)) {}
+              m_updateStartDate(QDate(1900,1,1)), m_timestampOffset(0) {}
 
   bool m_valid;
   enum NamePreference {
@@ -77,6 +80,7 @@ public:
   KOnlineBankingStatus* m_statusDlg;
   Wallet *m_wallet;
   QDate m_updateStartDate;
+  int m_timestampOffset;
 };
 
 
@@ -93,8 +97,8 @@ OFXImporter::OFXImporter(QObject *parent, const QVariantList &args) :
     d(new Private)
 {
   Q_UNUSED(args)
-  setComponentName("ofximporter", i18n("OFX Importer"));
-  setXMLFile("ofximporter.rc");
+  setComponentName(QStringLiteral("ofximporter"), i18n("OFX Importer"));
+  setXMLFile(QStringLiteral("ofximporter.rc"));
   createActions();
 
   // For ease announce that we have been loaded.
@@ -107,16 +111,13 @@ OFXImporter::~OFXImporter()
   qDebug("Plugins: ofximporter unloaded");
 }
 
-void OFXImporter::injectExternalSettings(KMyMoneySettings* p)
-{
-  KMyMoneyGlobalSettings::injectExternalSettings(p);
-}
-
 void OFXImporter::createActions()
 {
-  QAction *action = actionCollection()->addAction("file_import_ofx");
-  action->setText(i18n("OFX..."));
-  connect(action, SIGNAL(triggered(bool)), this, SLOT(slotImportFile()));
+  const auto &kpartgui = QStringLiteral("file_import_ofx");
+  auto a = actionCollection()->addAction(kpartgui);
+  a->setText(i18n("OFX..."));
+  connect(a, &QAction::triggered, this, static_cast<void (OFXImporter::*)()>(&OFXImporter::slotImportFile));
+  connect(viewInterface(), &KMyMoneyPlugin::ViewInterface::viewStateChanged, action(qPrintable(kpartgui)), &QAction::setEnabled);
 }
 
 void OFXImporter::slotImportFile()
@@ -126,32 +127,36 @@ void OFXImporter::slotImportFile()
   option->setupUi(widget);
 
   QUrl url = importInterface()->selectFile(i18n("OFX import file selection"),
-             "",
-             "*.ofx *.qfx *.ofc|OFX files (*.ofx *.qfx *.ofc);;*|All files (*)",
+             QString(),
+             QStringLiteral("*.ofx *.qfx *.ofc|OFX files (*.ofx *.qfx *.ofc);;*|All files (*)"),
              QFileDialog::ExistingFile,
              widget);
 
   d->m_preferName = static_cast<OFXImporter::Private::NamePreference>(option->m_preferName->currentIndex());
 
   if (url.isValid()) {
-    if (isMyFormat(url.path())) {
-      slotImportFile(url.path());
+    const QString filename(url.toLocalFile());
+    if (isMyFormat(filename)) {
+      statementInterface()->resetMessages();
+      slotImportFile(filename);
+      statementInterface()->showMessages(d->m_statementlist.count());
+
     } else {
       KMessageBox::error(0, i18n("Unable to import %1 using the OFX importer plugin.  This file is not the correct format.", url.toDisplayString()), i18n("Incorrect format"));
     }
-
   }
+  delete option;
   delete widget;
 }
 
 QString OFXImporter::formatName() const
 {
-  return "OFX";
+  return QStringLiteral("OFX");
 }
 
 QString OFXImporter::formatFilenameFilter() const
 {
-  return "*.ofx *.qfx *.ofc";
+  return QStringLiteral("*.ofx *.qfx *.ofc");
 }
 
 
@@ -170,14 +175,16 @@ bool OFXImporter::isMyFormat(const QString& filename) const
     while (!ts.atEnd() && !result  && lineCount != 0) {
       // get a line of data and remove all unnecessary whitepace chars
       QString line = ts.readLine().simplified();
-      if (line.contains("<OFX>", Qt::CaseInsensitive)
-          || line.contains("<OFC>", Qt::CaseInsensitive))
+      if (line.contains(QStringLiteral("<OFX>"), Qt::CaseInsensitive)
+          || line.contains(QStringLiteral("<OFC>"), Qt::CaseInsensitive))
         result = true;
       // count only lines that contain some non white space chars
       if (!line.isEmpty())
         lineCount--;
     }
     f.close();
+  } else {
+    qDebug() << "OFXImporter::isMyFormat: unable to open" << filename << "with" << f.errorString();
   }
 
   return result;
@@ -214,6 +221,10 @@ bool OFXImporter::import(const QString& filename)
   LibofxContextPtr ctx = libofx_get_new_context();
   Q_CHECK_PTR(ctx);
 
+  // Don't show the position that caused a message to be shown
+  // This has no setter (see libofx.h)
+  ofx_show_position = false;
+
   qDebug("setup callback routines");
   ofx_set_transaction_cb(ctx, ofxTransactionCallback, this);
   ofx_set_statement_cb(ctx, ofxStatementCallback, this);
@@ -222,6 +233,7 @@ bool OFXImporter::import(const QString& filename)
   ofx_set_status_cb(ctx, ofxStatusCallback, this);
   qDebug("process data");
   libofx_proc_file(ctx, filename_deep, AUTODETECT);
+  qDebug("process data done");
   libofx_free_context(ctx);
 
   if (d->m_valid) {
@@ -235,7 +247,7 @@ QString OFXImporter::lastError() const
 {
   if (d->m_errors.count() == 0)
     return d->m_fatalerror;
-  return d->m_errors.join("<p>");
+  return d->m_errors.join(QStringLiteral("<p>"));
 }
 
 /* __________________________________________________________________________
@@ -257,11 +269,11 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
 
   if (data.date_posted_valid) {
     QDateTime dt;
-    dt.setTime_t(data.date_posted);
+    dt.setTime_t(data.date_posted - pofx->d->m_timestampOffset * 60);
     t.m_datePosted = dt.date();
   } else if (data.date_initiated_valid) {
     QDateTime dt;
-    dt.setTime_t(data.date_initiated);
+    dt.setTime_t(data.date_initiated - pofx->d->m_timestampOffset * 60);
     t.m_datePosted = dt.date();
   }
   if (t.m_datePosted.isValid()) {
@@ -281,9 +293,9 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
   }
 
   if (data.fi_id_valid) {
-    t.m_strBankID = QString("ID ") + QString::fromUtf8(data.fi_id);
+    t.m_strBankID = QStringLiteral("ID ") + QString::fromUtf8(data.fi_id);
   } else if (data.reference_number_valid) {
-    t.m_strBankID = QString("REF ") + QString::fromUtf8(data.reference_number);
+    t.m_strBankID = QStringLiteral("REF ") + QString::fromUtf8(data.reference_number);
   }
 
   // Decide whether to use NAME, PAYEEID or MEMO to construct the payee
@@ -418,31 +430,31 @@ int OFXImporter::ofxTransactionCallback(struct OfxTransactionData data, void * p
         //
       case OFX_CLOSUREOPT:
         unhandledtype = true;
-        type = "CLOSUREOPT (Close a position for an option)";
+        type = QStringLiteral("CLOSUREOPT (Close a position for an option)");
         break;
       case OFX_INVEXPENSE:
         unhandledtype = true;
-        type = "INVEXPENSE (Misc investment expense that is associated with a specific security)";
+        type = QStringLiteral("INVEXPENSE (Misc investment expense that is associated with a specific security)");
         break;
       case OFX_JRNLFUND:
         unhandledtype = true;
-        type = "JRNLFUND (Journaling cash holdings between subaccounts within the same investment account)";
+        type = QStringLiteral("JRNLFUND (Journaling cash holdings between subaccounts within the same investment account)");
         break;
       case OFX_MARGININTEREST:
         unhandledtype = true;
-        type = "MARGININTEREST (Margin interest expense)";
+        type = QStringLiteral("MARGININTEREST (Margin interest expense)");
         break;
       case OFX_RETOFCAP:
         unhandledtype = true;
-        type = "RETOFCAP (Return of capital)";
+        type = QStringLiteral("RETOFCAP (Return of capital)");
         break;
       case OFX_SPLIT:
         unhandledtype = true;
-        type = "SPLIT (Stock or mutial fund split)";
+        type = QStringLiteral("SPLIT (Stock or mutial fund split)");
         break;
       case OFX_TRANSFER:
         unhandledtype = true;
-        type = "TRANSFER (Transfer holdings in and out of the investment account)";
+        type = QStringLiteral("TRANSFER (Transfer holdings in and out of the investment account)");
         break;
       default:
         unhandledtype = true;
@@ -499,13 +511,13 @@ int OFXImporter::ofxStatementCallback(struct OfxStatementData data, void* pv)
 
   if (data.date_start_valid) {
     QDateTime dt;
-    dt.setTime_t(data.date_start);
+    dt.setTime_t(data.date_start - pofx->d->m_timestampOffset * 60);
     s.m_dateBegin = dt.date();
   }
 
   if (data.date_end_valid) {
     QDateTime dt;
-    dt.setTime_t(data.date_end);
+    dt.setTime_t(data.date_end - pofx->d->m_timestampOffset * 60);
     s.m_dateEnd = dt.date();
   }
 
@@ -566,7 +578,7 @@ int OFXImporter::ofxAccountCallback(struct OfxAccountData data, void * pv)
   }
 
   // ask KMyMoney for an account id
-  s.m_accountId = pofx->account("kmmofx-acc-ref", QString("%1-%2").arg(s.m_strRoutingNumber, s.m_strAccountNumber)).id();
+  s.m_accountId = pofx->account(QStringLiteral("kmmofx-acc-ref"), QString("%1-%2").arg(s.m_strRoutingNumber, s.m_strAccountNumber)).id();
 
   // copy over the securities
   s.m_listSecurities = pofx->d->m_securitylist;
@@ -632,7 +644,7 @@ int OFXImporter::ofxStatusCallback(struct OfxStatusData data, void * pv)
         break;
       default:
         pofx->addWarning(message);
-        pofx->addWarning("Previous message was an unknown type.  'WARNING' was assumed.");
+        pofx->addWarning(QStringLiteral("Previous message was an unknown type.  'WARNING' was assumed."));
         break;
     }
   }
@@ -642,10 +654,10 @@ int OFXImporter::ofxStatusCallback(struct OfxStatusData data, void * pv)
   return 0;
 }
 
-bool OFXImporter::importStatement(const MyMoneyStatement& s)
+QStringList OFXImporter::importStatement(const MyMoneyStatement &s)
 {
   qDebug("OfxImporterPlugin::importStatement start");
-  return statementInterface()->import(s);
+  return statementInterface()->import(s, false);
 }
 
 MyMoneyAccount OFXImporter::account(const QString& key, const QString& value) const
@@ -656,7 +668,7 @@ MyMoneyAccount OFXImporter::account(const QString& key, const QString& value) co
 void OFXImporter::protocols(QStringList& protocolList) const
 {
   protocolList.clear();
-  protocolList << "OFX";
+  protocolList << QStringLiteral("OFX");
 }
 
 QWidget* OFXImporter::accountConfigTab(const MyMoneyAccount& acc, QString& name)
@@ -670,17 +682,17 @@ MyMoneyKeyValueContainer OFXImporter::onlineBankingSettings(const MyMoneyKeyValu
 {
   MyMoneyKeyValueContainer kvp(current);
   // keep the provider name in sync with the one found in kmm_ofximport.desktop
-  kvp["provider"] = QLatin1String("OFX Importer");
+  kvp[QStringLiteral("provider")] = objectName().toLower();
   if (d->m_statusDlg) {
-    kvp.deletePair("appId");
-    kvp.deletePair("kmmofx-headerVersion");
-    kvp.deletePair("password");
+    kvp.deletePair(QStringLiteral("appId"));
+    kvp.deletePair(QStringLiteral("kmmofx-headerVersion"));
+    kvp.deletePair(QStringLiteral("password"));
 
     d->m_wallet = openSynchronousWallet();
     if (d->m_wallet && (d->m_wallet->hasFolder(KWallet::Wallet::PasswordFolder()) ||
                         d->m_wallet->createFolder(KWallet::Wallet::PasswordFolder())) &&
         d->m_wallet->setFolder(KWallet::Wallet::PasswordFolder())) {
-      QString key = OFX_PASSWORD_KEY(kvp.value("url"), kvp.value("uniqueId"));
+      QString key = OFX_PASSWORD_KEY(kvp.value(QStringLiteral("url")), kvp.value(QStringLiteral("uniqueId")));
       if (d->m_statusDlg->m_storePassword->isChecked()) {
         d->m_wallet->writePassword(key, d->m_statusDlg->m_password->text());
       } else {
@@ -690,25 +702,35 @@ MyMoneyKeyValueContainer OFXImporter::onlineBankingSettings(const MyMoneyKeyValu
       }
     } else {
       if (d->m_statusDlg->m_storePassword->isChecked()) {
-        kvp.setValue("password", d->m_statusDlg->m_password->text());
+        kvp.setValue(QStringLiteral("password"), d->m_statusDlg->m_password->text());
       }
     }
 
     if (!d->m_statusDlg->appId().isEmpty())
-      kvp.setValue("appId", d->m_statusDlg->appId());
-    kvp.setValue("kmmofx-headerVersion", d->m_statusDlg->headerVersion());
-    kvp.setValue("kmmofx-numRequestDays", QString::number(d->m_statusDlg->m_numdaysSpin->value()));
-    kvp.setValue("kmmofx-todayMinus", QString::number(d->m_statusDlg->m_todayRB->isChecked()));
-    kvp.setValue("kmmofx-lastUpdate", QString::number(d->m_statusDlg->m_lastUpdateRB->isChecked()));
-    kvp.setValue("kmmofx-pickDate", QString::number(d->m_statusDlg->m_pickDateRB->isChecked()));
-    kvp.setValue("kmmofx-specificDate", d->m_statusDlg->m_specificDate->date().toString());
-    kvp.setValue("kmmofx-preferName", QString::number(d->m_statusDlg->m_preferredPayee->currentIndex()));
+      kvp.setValue(QStringLiteral("appId"), d->m_statusDlg->appId());
+    kvp.setValue(QStringLiteral("kmmofx-headerVersion"), d->m_statusDlg->headerVersion());
+    kvp.setValue(QStringLiteral("kmmofx-numRequestDays"), QString::number(d->m_statusDlg->m_numdaysSpin->value()));
+    kvp.setValue(QStringLiteral("kmmofx-todayMinus"), QString::number(d->m_statusDlg->m_todayRB->isChecked()));
+    kvp.setValue(QStringLiteral("kmmofx-lastUpdate"), QString::number(d->m_statusDlg->m_lastUpdateRB->isChecked()));
+    kvp.setValue(QStringLiteral("kmmofx-pickDate"), QString::number(d->m_statusDlg->m_pickDateRB->isChecked()));
+    kvp.setValue(QStringLiteral("kmmofx-specificDate"), d->m_statusDlg->m_specificDate->date().toString());
+    kvp.setValue(QStringLiteral("kmmofx-preferName"), QString::number(d->m_statusDlg->m_preferredPayee->currentIndex()));
     if (!d->m_statusDlg->m_clientUidEdit->text().isEmpty())
-      kvp.setValue("clientUid", d->m_statusDlg->m_clientUidEdit->text());
+      kvp.setValue(QStringLiteral("clientUid"), d->m_statusDlg->m_clientUidEdit->text());
     else
-      kvp.deletePair("clientUid");
+      kvp.deletePair(QStringLiteral("clientUid"));
+    if (d->m_statusDlg->m_timestampOffset->time().msecsSinceStartOfDay() == 0) {
+      kvp.deletePair(QStringLiteral("kmmofx-timestampOffset"));
+    } else {
+      // get offset in minutes
+      int offset = d->m_statusDlg->m_timestampOffset->time().msecsSinceStartOfDay() / 1000 / 60;
+      if (d->m_statusDlg->m_timestampOffsetSign->currentText() == QStringLiteral("-")) {
+        offset = -offset;
+      }
+      kvp.setValue(QStringLiteral("kmmofx-timestampOffset"), QString::number(offset));
+    }
     // get rid of pre 4.6 values
-    kvp.deletePair("kmmofx-preferPayeeid");
+    kvp.deletePair(QStringLiteral("kmmofx-preferPayeeid"));
   }
   return kvp;
 }
@@ -738,31 +760,31 @@ bool OFXImporter::updateAccount(const MyMoneyAccount& acc, bool moreAccounts)
   try {
     if (!acc.id().isEmpty()) {
       // Save the value of preferName to be used by ofxTransactionCallback
-      d->m_preferName = static_cast<OFXImporter::Private::NamePreference>(acc.onlineBankingSettings().value("kmmofx-preferName").toInt());
+      d->m_preferName = static_cast<OFXImporter::Private::NamePreference>(acc.onlineBankingSettings().value(QStringLiteral("kmmofx-preferName")).toInt());
       QPointer<KOfxDirectConnectDlg> dlg = new KOfxDirectConnectDlg(acc);
 
-      connect(dlg, SIGNAL(statementReady(QString)),
-              this, SLOT(slotImportFile(QString)));
+      connect(dlg.data(), &KOfxDirectConnectDlg::statementReady, this, static_cast<void (OFXImporter::*)(const QString &)>(&OFXImporter::slotImportFile));
 
       // get the date of the earliest transaction that we are interested in
       // from the settings for this account
       MyMoneyKeyValueContainer settings = acc.onlineBankingSettings();
-      if (!settings.value("provider").isEmpty()) {
-        if ((settings.value("kmmofx-todayMinus").toInt() != 0) && !settings.value("kmmofx-numRequestDays").isEmpty()) {
+      if (!settings.value(QStringLiteral("provider")).isEmpty()) {
+        if ((settings.value(QStringLiteral("kmmofx-todayMinus")).toInt() != 0) && !settings.value(QStringLiteral("kmmofx-numRequestDays")).isEmpty()) {
           //kDebug(0) << "start date = today minus";
-          d->m_updateStartDate = QDate::currentDate().addDays(-settings.value("kmmofx-numRequestDays").toInt());
-        } else if ((settings.value("kmmofx-lastUpdate").toInt() != 0) && !acc.value("lastImportedTransactionDate").isEmpty()) {
+          d->m_updateStartDate = QDate::currentDate().addDays(-settings.value(QStringLiteral("kmmofx-numRequestDays")).toInt());
+        } else if ((settings.value(QStringLiteral("kmmofx-lastUpdate")).toInt() != 0) && !acc.value(QStringLiteral("lastImportedTransactionDate")).isEmpty()) {
           //kDebug(0) << "start date = last update";
-          d->m_updateStartDate = QDate::fromString(acc.value("lastImportedTransactionDate"), Qt::ISODate);
-        } else if ((settings.value("kmmofx-pickDate").toInt() != 0) && !settings.value("kmmofx-specificDate").isEmpty()) {
+          d->m_updateStartDate = QDate::fromString(acc.value(QStringLiteral("lastImportedTransactionDate")), Qt::ISODate);
+        } else if ((settings.value(QStringLiteral("kmmofx-pickDate")).toInt() != 0) && !settings.value(QStringLiteral("kmmofx-specificDate")).isEmpty()) {
           //kDebug(0) << "start date = pick date";
-          d->m_updateStartDate = QDate::fromString(settings.value("kmmofx-specificDate"));
+          d->m_updateStartDate = QDate::fromString(settings.value(QStringLiteral("kmmofx-specificDate")));
         }
         else {
           //kDebug(0) << "start date = today - 2 months";
           d->m_updateStartDate = QDate::currentDate().addMonths(-2);
         }
       }
+      d->m_timestampOffset = settings.value("kmmofx-timestampOffset").toInt();
       //kDebug(0) << "ofx plugin: account" << acc.name() << "earliest transaction date to process =" << qPrintable(d->m_updateStartDate.toString(Qt::ISODate));
 
       if (dlg->init())
@@ -771,10 +793,10 @@ bool OFXImporter::updateAccount(const MyMoneyAccount& acc, bool moreAccounts)
 
       // reset the earliest-interesting-transaction date to the non-specific account setting
       d->m_updateStartDate = QDate(1900,1,1);
-
+      d->m_timestampOffset = 0;
     }
   } catch (const MyMoneyException &e) {
-    KMessageBox::information(0 , i18n("Error connecting to bank: %1", e.what()));
+    KMessageBox::information(0 , i18n("Error connecting to bank: %1", QString::fromLatin1(e.what())));
   }
 
   return false;
@@ -788,11 +810,12 @@ void OFXImporter::slotImportFile(const QString& url)
   }
 }
 
-bool OFXImporter::storeStatements(QList<MyMoneyStatement>& statements)
+bool OFXImporter::storeStatements(const QList<MyMoneyStatement> &statements)
 {
-  bool hasstatements = (statements.count() > 0);
-  bool ok = true;
-  bool abort = false;
+  if (statements.isEmpty())
+    return true;
+  auto ok = true;
+  auto abort = false;
 
   // FIXME Deal with warnings/errors coming back from plugins
   /*if ( ofx.errors().count() )
@@ -807,18 +830,19 @@ bool OFXImporter::storeStatements(QList<MyMoneyStatement>& statements)
       abort = true;
   }*/
 
-  qDebug("OfxImporterPlugin::storeStatements() with %d statements called", static_cast<int>(statements.count()));
-  QList<MyMoneyStatement>::const_iterator it_s = statements.constBegin();
-  while (it_s != statements.constEnd() && !abort) {
-    ok = ok && importStatement((*it_s));
-    ++it_s;
+  qDebug("OfxImporterPlugin::storeStatements() with %d statements called", statements.count());
+
+  for (const auto& statement : statements) {
+    if (abort)
+      break;
+    if (importStatement(statement).isEmpty())
+      ok = false;
   }
 
-  if (hasstatements && !ok) {
-    KMessageBox::error(0, i18n("Importing process terminated unexpectedly."), i18n("Failed to import all statements."));
-  }
+  if (!ok)
+    KMessageBox::error(nullptr, i18n("Importing process terminated unexpectedly."), i18n("Failed to import all statements."));
 
-  return (!hasstatements || ok);
+  return ok;
 }
 
 void OFXImporter::addnew()
