@@ -42,7 +42,6 @@
 #include "mymoneyutils.h"
 #include "viewinterface.h"
 #include "mymoneyenums.h"
-#include "kmymoneyglobalsettings.h"
 
 #include "kreconciliationreportdlg.h"
 
@@ -58,11 +57,6 @@ ReconciliationReport::~ReconciliationReport()
   qDebug("Plugins: reconciliation report unloaded");
 }
 
-void ReconciliationReport::injectExternalSettings(KMyMoneySettings* p)
-{
-  KMyMoneyGlobalSettings::injectExternalSettings(p);
-}
-
 void ReconciliationReport::plug()
 {
   connect(viewInterface(), &KMyMoneyPlugin::ViewInterface::accountReconciled, this, &ReconciliationReport::slotGenerateReconciliationReport);
