@@ -16,21 +16,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
-
-/****************************************************************************
-  Contains code from the func_xirr and related methods of financial.cpp
-  - KOffice 1.6 by Sascha Pfau.  Sascha agreed to relicense those methods under
-  GPLv2 or later.
-*****************************************************************************/
-
 #include "querytable.h"
 
 #include <cmath>
@@ -49,6 +34,7 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include "cashflowlist.h"
 #include "mymoneyfile.h"
 #include "mymoneyaccount.h"
 #include "mymoneysecurity.h"
@@ -67,181 +53,6 @@
 namespace reports
 {
 
-// ****************************************************************************
-//
-// CashFlowListItem implementation
-//
-//   Cash flow analysis tools for investment reports
-//
-// ****************************************************************************
-
-QDate CashFlowListItem::m_sToday = QDate::currentDate();
-
-MyMoneyMoney CashFlowListItem::NPV(double _rate) const
-{
-  double T = static_cast<double>(m_sToday.daysTo(m_date)) / 365.0;
-  MyMoneyMoney result(m_value.toDouble() / pow(1 + _rate, T), 100);
-
-  //qDebug() << "CashFlowListItem::NPV( " << _rate << " ) == " << result;
-
-  return result;
-}
-
-// ****************************************************************************
-//
-// CashFlowList implementation
-//
-//   Cash flow analysis tools for investment reports
-//
-// ****************************************************************************
-
-CashFlowListItem CashFlowList::mostRecent() const
-{
-  CashFlowList dupe(*this);
-
-  qSort(dupe);
-
-  //qDebug() << " CashFlowList::mostRecent() == " << dupe.back().date().toString(Qt::ISODate);
-
-  return dupe.back();
-}
-
-MyMoneyMoney CashFlowList::NPV(double _rate) const
-{
-  MyMoneyMoney result;
-
-  const_iterator it_cash = constBegin();
-  while (it_cash != constEnd()) {
-    result += (*it_cash).NPV(_rate);
-    ++it_cash;
-  }
-
-  //qDebug() << "CashFlowList::NPV( " << _rate << " ) == " << result << "------------------------" << endl;
-
-  return result;
-}
-
-double CashFlowList::calculateXIRR() const
-{
-  double resultRate = 0.00001;
-
-  double resultZero = 0.00000;
-  //if ( args.count() > 2 )
-  //  resultRate = calc->conv()->asFloat ( args[2] ).asFloat();
-
-// check pairs and count >= 2 and guess > -1.0
-  //if ( args[0].count() != args[1].count() || args[1].count() < 2 || resultRate <= -1.0 )
-  //  return Value::errorVALUE();
-
-// define max epsilon
-  static const double maxEpsilon = 1e-5;
-
-// max number of iterations
-  static const int maxIter = 50;
-
-// Newton's method - try to find a res, with a accuracy of maxEpsilon
-  double rateEpsilon, newRate, resultValue;
-  int i = 0;
-  bool contLoop;
-
-  do {
-    resultValue = xirrResult(resultRate);
-
-    double resultDerive = xirrResultDerive(resultRate);
-
-    //check what happens if xirrResultDerive is zero
-    //Don't know if it is correct to dismiss the result
-    if (resultDerive != 0) {
-      newRate =  resultRate - resultValue / resultDerive;
-    } else {
-
-      newRate =  resultRate - resultValue;
-    }
-
-    rateEpsilon = fabs(newRate - resultRate);
-
-    resultRate = newRate;
-    contLoop = (rateEpsilon > maxEpsilon) && (fabs(resultValue) > maxEpsilon);
-  } while (contLoop && (++i < maxIter));
-
-  if (contLoop)
-    return resultZero;
-
-  return resultRate;
-}
-
-double CashFlowList::xirrResult(double& rate) const
-{
-  double r = rate + 1.0;
-  double res = 0.00000;//back().value().toDouble();
-
-  QList<CashFlowListItem>::const_iterator list_it = constBegin();
-  while (list_it != constEnd()) {
-    double e_i = ((* list_it).today().daysTo((* list_it).date())) / 365.0;
-    MyMoneyMoney val = (* list_it).value();
-
-    if (e_i < 0) {
-      res += val.toDouble() * pow(r, -e_i);
-    } else {
-      res += val.toDouble() / pow(r, e_i);
-    }
-    ++list_it;
-  }
-
-  return res;
-}
-
-
-double CashFlowList::xirrResultDerive(double& rate) const
-{
-  double r = rate + 1.0;
-  double res = 0.00000;
-
-  QList<CashFlowListItem>::const_iterator list_it = constBegin();
-  while (list_it != constEnd()) {
-    double e_i = ((* list_it).today().daysTo((* list_it).date())) / 365.0;
-    MyMoneyMoney val = (* list_it).value();
-
-    res -= e_i * val.toDouble() / pow(r, e_i + 1.0);
-    ++list_it;
-  }
-
-  return res;
-}
-
-double CashFlowList::IRR() const
-{
-  double result = 0.0;
-
-  // set 'today', which is the most recent of all dates in the list
-  CashFlowListItem::setToday(mostRecent().date());
-
-  result = calculateXIRR();
-  return result;
-}
-
-MyMoneyMoney CashFlowList::total() const
-{
-  MyMoneyMoney result;
-
-  const_iterator it_cash = constBegin();
-  while (it_cash != constEnd()) {
-    result += (*it_cash).value();
-    ++it_cash;
-  }
-
-  return result;
-}
-
-void CashFlowList::dumpDebug() const
-{
-  const_iterator it_item = constBegin();
-  while (it_item != constEnd()) {
-    qDebug() << (*it_item).date().toString(Qt::ISODate) << " " << (*it_item).value().toString();
-    ++it_item;
-  }
-}
-
 // ****************************************************************************
 //
 // QueryTable implementation
@@ -546,10 +357,12 @@ void QueryTable::constructTotalRows()
               if (subtotal == ctReturnInvestment)
                 totalsRow[subtotal] = helperROI((*currencyGrp).at(i + 1).value(ctBuys) - (*currencyGrp).at(i + 1).value(ctReinvestIncome), (*currencyGrp).at(i + 1).value(ctSells),
                                                 (*currencyGrp).at(i + 1).value(ctStartingBalance), (*currencyGrp).at(i + 1).value(ctEndingBalance) + (*currencyGrp).at(i + 1).value(ctMarketValue),
-                                                (*currencyGrp).at(i + 1).value(ctCashIncome)).toString();
-              else if (subtotal == ctPercentageGain)
-                totalsRow[subtotal] = (((*currencyGrp).at(i + 1).value(ctBuys) + (*currencyGrp).at(i + 1).value(ctMarketValue)) / (*currencyGrp).at(i + 1).value(ctBuys).abs()).toString();
-              else if (subtotal == ctPrice)
+                                                (*currencyGrp).at(i + 1).value(ctCashIncome));
+              else if (subtotal == ctPercentageGain) {
+                const MyMoneyMoney denominator = (*currencyGrp).at(i + 1).value(ctBuys).abs();
+                totalsRow[subtotal] = denominator.isZero() ? QString():
+                    (((*currencyGrp).at(i + 1).value(ctBuys) + (*currencyGrp).at(i + 1).value(ctMarketValue)) / denominator).toString();
+              } else if (subtotal == ctPrice)
                 totalsRow[subtotal] = MyMoneyMoney((*currencyGrp).at(i + 1).value(ctPrice) / (*currencyGrp).at(i + 1).value(ctRowsCount)).toString();
             }
 
@@ -606,7 +419,7 @@ void QueryTable::constructTotalRows()
           if (subtotal == ctReturnInvestment)
             totalsRow[subtotal] = helperROI((*currencyGrp).at(0).value(ctBuys) - (*currencyGrp).at(0).value(ctReinvestIncome), (*currencyGrp).at(0).value(ctSells),
                                             (*currencyGrp).at(0).value(ctStartingBalance), (*currencyGrp).at(0).value(ctEndingBalance) + (*currencyGrp).at(0).value(ctMarketValue),
-                                            (*currencyGrp).at(0).value(ctCashIncome)).toString();
+                                            (*currencyGrp).at(0).value(ctCashIncome));
           else if (subtotal == ctPercentageGain)
             totalsRow[subtotal] = (((*currencyGrp).at(0).value(ctBuys) + (*currencyGrp).at(0).value(ctMarketValue)) / (*currencyGrp).at(0).value(ctBuys).abs()).toString();
           else if (subtotal == ctPrice)
@@ -1243,31 +1056,25 @@ void QueryTable::constructTransactionTable()
   }
 }
 
-MyMoneyMoney QueryTable::helperROI(const MyMoneyMoney &buys, const MyMoneyMoney &sells, const MyMoneyMoney &startingBal, const MyMoneyMoney &endingBal, const MyMoneyMoney &cashIncome) const
+QString QueryTable::helperROI(const MyMoneyMoney &buys, const MyMoneyMoney &sells, const MyMoneyMoney &startingBal, const MyMoneyMoney &endingBal, const MyMoneyMoney &cashIncome) const
 {
   MyMoneyMoney returnInvestment;
-  if (!buys.isZero() || !startingBal.isZero()) {
+  if (!(startingBal - buys).isZero()) {
     returnInvestment = (sells + buys + cashIncome + endingBal - startingBal) / (startingBal - buys);
-    returnInvestment = returnInvestment.convert(10000);
+    return returnInvestment.convert(10000).toString();
   } else
-    returnInvestment = MyMoneyMoney(); // if no investment then no return on investment
-  return returnInvestment;
+    return QString();
 }
 
-MyMoneyMoney QueryTable::helperIRR(const CashFlowList &all) const
+QString QueryTable::helperIRR(const CashFlowList &all) const
 {
-  MyMoneyMoney annualReturn;
   try {
-    double irr = all.IRR();
-#ifdef Q_CC_MSVC
-    annualReturn = MyMoneyMoney(_isnan(irr) ? 0 : irr, 10000);
-#else
-    annualReturn = MyMoneyMoney(std::isnan(irr) ? 0 : irr, 10000);
-#endif
-  } catch (QString e) {
-    qDebug() << e;
+    return MyMoneyMoney(all.XIRR(), 10000).toString();
+  } catch (MyMoneyException &e) {
+    qDebug() << e.what();
+    all.dumpDebug();
+    return QString();
   }
-  return annualReturn;
 }
 
 void QueryTable::sumInvestmentValues(const ReportAccount& account, QList<CashFlowList>& cfList, QList<MyMoneyMoney>& shList) const
@@ -1598,12 +1405,9 @@ void QueryTable::constructPerformanceRow(const ReportAccount& account, TableRow&
     break;
   }
 
-  MyMoneyMoney returnInvestment = helperROI(buysTotal - reinvestIncomeTotal, sellsTotal, startingBal, endingBal, cashIncomeTotal);
-  MyMoneyMoney annualReturn = helperIRR(all);
-
   result[ctBuys] = buysTotal.toString();
-  result[ctReturn] = annualReturn.toString();
-  result[ctReturnInvestment] = returnInvestment.toString();
+  result[ctReturn] = helperIRR(all);
+  result[ctReturnInvestment] = helperROI(buysTotal - reinvestIncomeTotal, sellsTotal, startingBal, endingBal, cashIncomeTotal);
   result[ctEquityType] = MyMoneySecurity::securityTypeToString(file->security(account.currencyId()).securityType());
 }
 
@@ -1654,7 +1458,8 @@ void QueryTable::constructCapitalGainRow(const ReportAccount& account, TableRow&
     result[ctLastPrice] = price.toString();
     result[ctMarketValue] = endingBal.toString();
     result[ctCapitalGain] = (buysTotal + endingBal).toString();
-    result[ctPercentageGain] = ((buysTotal + endingBal)/buysTotal.abs()).toString();
+    result[ctPercentageGain] = buysTotal.isZero() ? QString() :
+        ((buysTotal + endingBal)/buysTotal.abs()).toString();
     break;
   }
   case eMyMoney::Report::InvestmentSum::Sold:
@@ -1793,7 +1598,7 @@ void QueryTable::constructAccountTable()
       // convert map of top accounts with cashflows to TableRow
       for (QMap<QString, CashFlowList>::iterator topAccount = (*currencyAccGrp).begin(); topAccount != (*currencyAccGrp).end(); ++topAccount) {
         qtotalsrow[ctTopAccount] = topAccount.key();
-        qtotalsrow[ctReturn] = helperIRR(topAccount.value()).toString();
+        qtotalsrow[ctReturn] = helperIRR(topAccount.value());
         qtotalsrow[ctCurrency] = currencyAccGrp.key();
         currencyGrandCashFlow[currencyAccGrp.key()] += topAccount.value();  // cumulative sum of cashflows of each topaccount
         m_rows.append(qtotalsrow);            // rows aren't sorted yet, so no problem with adding them randomly at the end
@@ -1803,7 +1608,7 @@ void QueryTable::constructAccountTable()
     QMap<QString, CashFlowList>::iterator currencyGrp = currencyGrandCashFlow.begin();
     qtotalsrow[ctTopAccount].clear();          // empty topaccount because it's grand cashflow
     while (currencyGrp != currencyGrandCashFlow.end()) {
-      qtotalsrow[ctReturn] = helperIRR(currencyGrp.value()).toString();
+      qtotalsrow[ctReturn] = helperIRR(currencyGrp.value());
       qtotalsrow[ctCurrency] = currencyGrp.key();
       m_rows.append(qtotalsrow);
       ++currencyGrp;
