@@ -86,6 +86,8 @@ XMLStorage::~XMLStorage()
 
 MyMoneyStorageMgr *XMLStorage::open(const QUrl &url)
 {
+  fileUrl.clear();
+
   if (url.scheme() == QLatin1String("sql"))
     return nullptr;
 
@@ -230,9 +232,18 @@ MyMoneyStorageMgr *XMLStorage::open(const QUrl &url)
     MyMoneyFile::instance()->detachStorage();
   }
 
+  fileUrl = url;
+  //write the directory used for this file as the default one for next time.
+  appInterface()->writeLastUsedDir(url.toDisplayString(QUrl::RemoveFilename | QUrl::PreferLocalFile | QUrl::StripTrailingSlash));
+
   return storage;
 }
 
+QUrl XMLStorage::openUrl() const
+{
+  return fileUrl;
+}
+
 bool XMLStorage::save(const QUrl &url)
 {
   QString filename = url.toLocalFile();
