@@ -643,10 +643,10 @@ public:
             }
 
             MyMoneyTransaction t(s.id(), KMyMoneyUtils::scheduledTransaction(s));
-            // if the transaction is scheduled and overdue, it can't
-            // certainly be posted in the past. So we take today's date
-            // as the alternative
-            if (s.isOverdue()) {
+            if (s.isOverdue() && !KMyMoneySettings::showPlannedScheduleDates()) {
+              // if the transaction is scheduled and overdue, it can't
+              // certainly be posted in the past. So we take today's date
+              // as the alternative
               t.setPostDate(s.adjustedDate(QDate::currentDate(), s.weekendOption()));
             } else {
               t.setPostDate(s.adjustedNextDueDate());
@@ -657,7 +657,7 @@ public:
               }
             }
             // keep track of this payment locally (not in the engine)
-            if (s.isOverdue()) {
+            if (s.isOverdue() && !KMyMoneySettings::showPlannedScheduleDates()) {
               s.setLastPayment(QDate::currentDate());
             } else {
               s.setLastPayment(s.nextDueDate());
@@ -1203,14 +1203,17 @@ public:
         }
         // need to ensure "nextCheckNumber" is still correct
         auto acc = file->account((*it_t).split().accountId());
+
         // the "lastNumberUsed" might have been the txn number deleted
         // so adjust it
         QString deletedNum = (*it_t).split().number();
-        // decrement deletedNum and set new "lastNumberUsed"
-        QString num = KMyMoneyUtils::getAdjacentNumber(deletedNum, -1);
-        acc.setValue("lastNumberUsed", num);
+        if (deletedNum == acc.value("lastNumberUsed")) {
+          // decrement deletedNum and set new "lastNumberUsed"
+          QString num = KMyMoneyUtils::getAdjacentNumber(deletedNum, -1);
+          acc.setValue("lastNumberUsed", num);
+          file->modifyAccount(acc);
+        }
 
-        file->modifyAccount(acc);
         list.erase(it_t);
         it_t = list.begin();
         emit q->slotStatusProgress(i++, 0);
