@@ -45,6 +45,7 @@
 #include <QWebEngineView>
 #else
 #include <KWebView>
+#include <QWebFrame>
 #endif
 
 // ----------------------------------------------------------------------------
@@ -64,15 +65,13 @@
 #include "mymoneyutils.h"
 #include "kmymoneyutils.h"
 #include "kwelcomepage.h"
-#include "kmymoneyglobalsettings.h"
+#include "kmymoneysettings.h"
 #include "mymoneyfile.h"
 #include "mymoneyaccount.h"
 #include "mymoneyprice.h"
+#include "mymoneyreport.h"
+#include "mymoneymoney.h"
 #include "mymoneyforecast.h"
-#include "kreportchartview.h"
-#include "pivottable.h"
-#include "pivotgrid.h"
-#include "reportaccount.h"
 #include "mymoneysplit.h"
 #include "mymoneytransaction.h"
 #include "icons.h"
@@ -80,8 +79,10 @@
 #include "mymoneyschedule.h"
 #include "mymoneysecurity.h"
 #include "mymoneyexception.h"
+#include "kmymoneyplugin.h"
 #include "mymoneyenums.h"
 #include "menuenums.h"
+#include "plugins/views/reports/reportsviewenums.h"
 
 #define VIEW_LEDGER         "ledger"
 #define VIEW_SCHEDULE       "schedule"
@@ -115,8 +116,6 @@ bool accountNameLess(const MyMoneyAccount &acc1, const MyMoneyAccount &acc2)
   return acc1.name().localeAwareCompare(acc2.name()) < 0;
 }
 
-using namespace reports;
-
 class KHomeViewPrivate : public KMyMoneyViewBasePrivate
 {
   Q_DECLARE_PUBLIC(KHomeView)
@@ -128,15 +127,16 @@ public:
     m_view(nullptr),
     m_showAllSchedules(false),
     m_needLoad(true),
-    m_netWorthGraphLastValidSize(400, 300)
+    m_netWorthGraphLastValidSize(400, 300),
+    m_currentPrinter(nullptr)
   {
   }
 
   ~KHomeViewPrivate() {
     // if user wants to remember the font size, store it here
-    if (KMyMoneyGlobalSettings::rememberZoomFactor() && m_view) {
-        KMyMoneyGlobalSettings::setZoomFactor(m_view->zoomFactor());
-        KMyMoneyGlobalSettings::self()->save();
+    if (KMyMoneySettings::rememberZoomFactor() && m_view) {
+        KMyMoneySettings::setZoomFactor(m_view->zoomFactor());
+        KMyMoneySettings::self()->save();
       }
   }
 
@@ -167,7 +167,6 @@ public:
 
     vbox->addWidget(m_view);
 
-    m_view->setHtml(KWelcomePage::welcomePage(), QUrl("file://"));
   #ifdef ENABLE_WEBENGINE
     q->connect(m_view->page(), &QWebEnginePage::urlChanged,
             q, &KHomeView::slotOpenUrl);
@@ -176,6 +175,8 @@ public:
     q->connect(m_view->page(), &KWebPage::linkClicked,
             q, &KHomeView::slotOpenUrl);
   #endif
+
+    q->connect(MyMoneyFile::instance(), &MyMoneyFile::dataChanged, q, &KHomeView::refresh);
   }
 
   /**
@@ -199,7 +200,7 @@ public:
 
     QString cellStatus, cellCounts, pathOK, pathTODO, pathNotOK;
 
-    if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) {
+    if (KMyMoneySettings::showBalanceStatusOfOnlineAccounts()) {
       //show account's online-status
       pathOK = QPixmapToDataUri(Icons::get(Icon::DialogOKApply).pixmap(QSize(16,16)));
       pathTODO = QPixmapToDataUri(Icons::get(Icon::MailReceive).pixmap(QSize(16,16)));
@@ -224,16 +225,16 @@ public:
     int countNotMarked = 0, countCleared = 0, countNotReconciled = 0;
     QString countStr;
 
-    if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions() || KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
-      countNotMarked = file->countTransactionsWithSpecificReconciliationState(acc.id(), TransactionFilter::State::NotReconciled);
+    if (KMyMoneySettings::showCountOfUnmarkedTransactions() || KMyMoneySettings::showCountOfNotReconciledTransactions())
+      countNotMarked = m_transactionStats[acc.id()][(int)Split::State::NotReconciled];
 
-    if (KMyMoneyGlobalSettings::showCountOfClearedTransactions() || KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
-      countCleared = file->countTransactionsWithSpecificReconciliationState(acc.id(), TransactionFilter::State::Cleared);
+    if (KMyMoneySettings::showCountOfClearedTransactions() || KMyMoneySettings::showCountOfNotReconciledTransactions())
+      countCleared = m_transactionStats[acc.id()][(int)Split::State::Cleared];
 
-    if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
+    if (KMyMoneySettings::showCountOfNotReconciledTransactions())
       countNotReconciled = countNotMarked + countCleared;
 
-    if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions()) {
+    if (KMyMoneySettings::showCountOfUnmarkedTransactions()) {
       if (countNotMarked)
         countStr = QString("%1").arg(countNotMarked);
       else
@@ -241,7 +242,7 @@ public:
       tmp += QString("<td class=\"center\">%1</td>").arg(countStr);
     }
 
-    if (KMyMoneyGlobalSettings::showCountOfClearedTransactions()) {
+    if (KMyMoneySettings::showCountOfClearedTransactions()) {
       if (countCleared)
         countStr = QString("%1").arg(countCleared);
       else
@@ -249,7 +250,7 @@ public:
       tmp += QString("<td class=\"center\">%1</td>").arg(countStr);
     }
 
-    if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions()) {
+    if (KMyMoneySettings::showCountOfNotReconciledTransactions()) {
       if (countNotReconciled)
         countStr = QString("%1").arg(countNotReconciled);
       else
@@ -257,6 +258,11 @@ public:
       tmp += QString("<td class=\"center\">%1</td>").arg(countStr);
     }
 
+    if (KMyMoneySettings::showDateOfLastReconciliation()) {
+      const auto lastReconciliationDate = acc.lastReconciliationDate().toString(Qt::SystemLocaleShortDate).replace(QChar(' '), "&nbsp;");
+      tmp += QString("<td>%1</d>").arg(lastReconciliationDate);
+    }
+
     //show account balance
     tmp += QString("<td class=\"right\">%1</td>").arg(showColoredAmount(amount, value.isNegative()));
 
@@ -276,11 +282,10 @@ public:
 
   void showAccountEntry(const MyMoneyAccount& acc)
   {
-    MyMoneyFile* file = MyMoneyFile::instance();
-    MyMoneySecurity currency = file->currency(acc.currencyId());
+    const auto file = MyMoneyFile::instance();
     MyMoneyMoney value;
 
-    bool showLimit = KMyMoneyGlobalSettings::showLimitInfo();
+    bool showLimit = KMyMoneySettings::showLimitInfo();
 
     if (acc.accountType() == Account::Type::Investment) {
       //investment accounts show the balances of all its subaccounts
@@ -292,9 +297,9 @@ public:
       //get balance for normal accounts
       value = file->balance(acc.id(), QDate::currentDate());
       if (acc.currencyId() != file->baseCurrency().id()) {
-        ReportAccount repAcc = ReportAccount(acc.id());
-        MyMoneyMoney curPrice = repAcc.baseCurrencyPrice(QDate::currentDate());
-        MyMoneyMoney baseValue = value * curPrice;
+        const auto curPrice = file->price(acc.tradingCurrencyId(), file->baseCurrency().id(), QDate::currentDate());
+        const auto curRate = curPrice.rate(file->baseCurrency().id());
+        auto baseValue = value * curRate;
         baseValue = baseValue.convert(file->baseCurrency().smallestAccountFraction());
         m_total += baseValue;
       } else {
@@ -356,7 +361,7 @@ public:
   {
     if (isNegative) {
       //if negative, get the settings for negative numbers
-      return QString("<font color=\"%1\">%2</font>").arg(KMyMoneyGlobalSettings::schemeColor(SchemeColor::Negative).name(), amount);
+      return QString("<font color=\"%1\">%2</font>").arg(KMyMoneySettings::schemeColor(SchemeColor::Negative).name(), amount);
     }
 
     //if positive, return the same string
@@ -372,7 +377,7 @@ public:
     m_accountList.clear();
 
     //reinitialize the object
-    m_forecast = KMyMoneyGlobalSettings::forecast();
+    m_forecast = KMyMoneyUtils::forecast();
 
     //If forecastDays lower than accountsCycle, adjust to the first cycle
     if (m_forecast.accountsCycle() > m_forecast.forecastDays())
@@ -407,13 +412,26 @@ public:
 
   void loadView()
   {
-    m_view->setZoomFactor(KMyMoneyGlobalSettings::zoomFactor());
+    m_view->setZoomFactor(KMyMoneySettings::zoomFactor());
 
     QList<MyMoneyAccount> list;
-    MyMoneyFile::instance()->accountList(list);
-    if (list.count() == 0) {
+    if (MyMoneyFile::instance()->storage()) {
+      MyMoneyFile::instance()->accountList(list);
+    }
+    if (list.isEmpty()) {
       m_view->setHtml(KWelcomePage::welcomePage(), QUrl("file://"));
     } else {
+      // preload transaction statistics
+      m_transactionStats = MyMoneyFile::instance()->countTransactionsWithSpecificReconciliationState();
+
+      // keep current location on page
+      int scrollBarPos = 0;
+#ifdef ENABLE_WEBENGINE
+        /// @todo cannot test this
+#else
+      scrollBarPos = m_view->page()->mainFrame()->scrollBarValue(Qt::Vertical);
+#endif
+
       //clear the forecast flag so it will be reloaded
       m_forecast.setForecastDone(false);
 
@@ -431,7 +449,7 @@ public:
 
       m_html += QString("<div id=\"summarytitle\">%1</div>").arg(i18n("Your Financial Summary"));
 
-      QStringList settings = KMyMoneyGlobalSettings::itemList();
+      QStringList settings = KMyMoneySettings::listOfItems();
 
       QStringList::ConstIterator it;
 
@@ -488,60 +506,44 @@ public:
       m_html += footer;
 
       m_view->setHtml(m_html, QUrl("file://"));
+
+      if (scrollBarPos) {
+#ifdef ENABLE_WEBENGINE
+        /// @todo cannot test this
+#else
+        m_view->page()->mainFrame()->setScrollBarValue(Qt::Vertical, scrollBarPos);
+#endif
+      }
     }
   }
 
   void showNetWorthGraph()
   {
     Q_Q(KHomeView);
-    m_html += QString("<div class=\"shadow\"><div class=\"displayblock\"><div class=\"summaryheader\">%1</div>\n<div class=\"gap\">&nbsp;</div>\n").arg(i18n("Net Worth Forecast"));
-
-    MyMoneyReport reportCfg = MyMoneyReport(
-                                MyMoneyReport::eAssetLiability,
-                                MyMoneyReport::eMonths,
-                                TransactionFilter::Date::UserDefined, // overridden by the setDateFilter() call below
-                                MyMoneyReport::eDetailTotal,
-                                i18n("Net Worth Forecast"),
-                                i18n("Generated Report"));
-
-    reportCfg.setChartByDefault(true);
-    reportCfg.setChartCHGridLines(false);
-    reportCfg.setChartSVGridLines(false);
-    reportCfg.setChartDataLabels(false);
-    reportCfg.setChartType(MyMoneyReport::eChartLine);
-    reportCfg.setIncludingSchedules(false);
-    reportCfg.addAccountGroup(Account::Type::Asset);
-    reportCfg.addAccountGroup(Account::Type::Liability);
-    reportCfg.setColumnsAreDays(true);
-    reportCfg.setConvertCurrency(true);
-    reportCfg.setIncludingForecast(true);
-    reportCfg.setDateFilter(QDate::currentDate(), QDate::currentDate().addDays(+ 90));
-    reports::PivotTable table(reportCfg);
-
-    reports::KReportChartView* chartWidget = new reports::KReportChartView(0);
-
-    table.drawChart(*chartWidget);
 
     // Adjust the size
     QSize netWorthGraphSize = q->size();
     netWorthGraphSize -= QSize(80, 30);
-    // consider the computed size valid only if it's smaller on both axes that the applications size
-//    if (netWorthGraphSize.width() < kmymoney->width() || netWorthGraphSize.height() < kmymoney->height()) {
-      m_netWorthGraphLastValidSize = netWorthGraphSize;
-//    }
-    chartWidget->resize(m_netWorthGraphLastValidSize);
-
-    //save the chart to an image
-    QString chart = QPixmapToDataUri(chartWidget->coordinatePlane()->parent()->grab());
+    m_netWorthGraphLastValidSize = netWorthGraphSize;
 
+    m_html += QString("<div class=\"shadow\"><div class=\"displayblock\"><div class=\"summaryheader\">%1</div>\n<div class=\"gap\">&nbsp;</div>\n").arg(i18n("Net Worth Forecast"));
     m_html += QString("<table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >");
     m_html += QString("<tr>");
-    m_html += QString("<td><center><img src=\"%1\" ALT=\"Networth\" width=\"100%\" ></center></td>").arg(chart);
+
+    if (const auto reportsPlugin = pPlugins.data.value(QStringLiteral("reportsview"), nullptr)) {
+      const auto variantReport = reportsPlugin->requestData(QString(), eWidgetPlugin::WidgetType::NetWorthForecast);
+      if (!variantReport.isNull()) {
+        auto report = variantReport.value<QWidget *>();
+        report->resize(m_netWorthGraphLastValidSize);
+        m_html += QString("<td><center><img src=\"%1\" ALT=\"Networth\" width=\"100%\" ></center></td>").arg(QPixmapToDataUri(report->grab()));
+        delete report;
+      }
+    } else {
+      m_html += QString("<td><center>%1</center></td>").arg(i18n("Enable reports plugin to see this chart."));
+    }
+
     m_html += QString("</tr>");
     m_html += QString("</table></div></div>");
-
-    //delete the widget since we no longer need it
-    delete chartWidget;
   }
 
   void showPayments()
@@ -843,7 +845,7 @@ public:
         }
       }
     } catch (const MyMoneyException &e) {
-      qDebug("Unable to display schedule entry: %s", qPrintable(e.what()));
+      qDebug("Unable to display schedule entry: %s", e.what());
     }
   }
 
@@ -935,7 +937,7 @@ public:
       m_html += "<table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
       m_html += "<tr class=\"item\">";
 
-      if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) {
+      if (KMyMoneySettings::showBalanceStatusOfOnlineAccounts()) {
         QString pathStatusHeader = QPixmapToDataUri(Icons::get(Icon::Download).pixmap(QSize(16,16)));
         m_html += QString("<td class=\"center\"><img src=\"%1\" border=\"0\"></td>").arg(pathStatusHeader);
       }
@@ -944,21 +946,24 @@ public:
       m_html += i18n("Account");
       m_html += "</td>";
 
-      if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions())
+      if (KMyMoneySettings::showCountOfUnmarkedTransactions())
         m_html += QString("<td class=\"center\">!M</td>");
 
-      if (KMyMoneyGlobalSettings::showCountOfClearedTransactions())
+      if (KMyMoneySettings::showCountOfClearedTransactions())
         m_html += QString("<td class=\"center\">C</td>");
 
-      if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
+      if (KMyMoneySettings::showCountOfNotReconciledTransactions())
         m_html += QString("<td class=\"center\">!R</td>");
 
+      if (KMyMoneySettings::showDateOfLastReconciliation())
+        m_html += QString("<td>%1</td>").arg(i18n("Last Reconciled"));
+
       m_html += "<td width=\"25%\" class=\"right\">";
       m_html += i18n("Current Balance");
       m_html += "</td>";
 
       //only show limit info if user chose to do so
-      if (KMyMoneyGlobalSettings::showLimitInfo()) {
+      if (KMyMoneySettings::showLimitInfo()) {
         m_html += "<td width=\"40%\" class=\"right\">";
         m_html += i18n("To Minimum Balance / Maximum Credit");
         m_html += "</td>";
@@ -974,11 +979,12 @@ public:
       }
       m_html += QString("<tr class=\"row-%1\">").arg(i++ & 0x01 ? "even" : "odd");
       QString amount = m_total.formatMoney(file->baseCurrency().tradingSymbol(), prec);
-      if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) m_html += "<td></td>";
+      if (KMyMoneySettings::showBalanceStatusOfOnlineAccounts()) m_html += "<td></td>";
       m_html += QString("<td class=\"right\"><b>%1</b></td>").arg(i18n("Total"));
-      if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions()) m_html += "<td></td>";
-      if (KMyMoneyGlobalSettings::showCountOfClearedTransactions()) m_html += "<td></td>";
-      if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions()) m_html += "<td></td>";
+      if (KMyMoneySettings::showCountOfUnmarkedTransactions()) m_html += "<td></td>";
+      if (KMyMoneySettings::showCountOfClearedTransactions()) m_html += "<td></td>";
+      if (KMyMoneySettings::showCountOfNotReconciledTransactions()) m_html += "<td></td>";
+      if (KMyMoneySettings::showDateOfLastReconciliation()) m_html += "<td></td>";
       m_html += QString("<td class=\"right\"><b>%1</b></td></tr>").arg(showColoredAmount(amount, m_total.isNegative()));
       m_html += "</table></div></div>";
     }
@@ -1034,11 +1040,11 @@ public:
     if (accList.count() > 0) {
       // sort the accounts by name
       qStableSort(accList.begin(), accList.end(), accountNameLess);
-      int i = 0;
+      auto i = 0;
 
-      int colspan = 1;
+      auto colspan = 1;
       //get begin day
-      int beginDay = QDate::currentDate().daysTo(m_forecast.beginForecastDate());
+      auto beginDay = QDate::currentDate().daysTo(m_forecast.beginForecastDate());
       //if begin day is today skip to next cycle
       if (beginDay == 0)
         beginDay = m_forecast.accountsCycle();
@@ -1049,7 +1055,7 @@ public:
       m_html += "<tr class=\"item\"><td class=\"left\" width=\"40%\">";
       m_html += i18n("Account");
       m_html += "</td>";
-      int colWidth = 55 / (m_forecast.forecastDays() / m_forecast.accountsCycle());
+      auto colWidth = 55 / (m_forecast.forecastDays() / m_forecast.accountsCycle());
       for (i = 0; (i*m_forecast.accountsCycle() + beginDay) <= m_forecast.forecastDays(); ++i) {
         m_html += QString("<td width=\"%1%\" class=\"right\">").arg(colWidth);
 
@@ -1070,8 +1076,8 @@ public:
         m_html += QString("<td width=\"40%\">") +
                      link(VIEW_LEDGER, QString("?id=%1").arg((*it_account).id())) + (*it_account).name() + linkend() + "</td>";
 
-        int dropZero = -1; //account dropped below zero
-        int dropMinimum = -1; //account dropped below minimum balance
+        qint64 dropZero = -1; //account dropped below zero
+        qint64 dropMinimum = -1; //account dropped below minimum balance
         QString minimumBalance = (*it_account).value("minimumBalance");
         MyMoneyMoney minBalance = MyMoneyMoney(minimumBalance);
         MyMoneySecurity currency;
@@ -1085,7 +1091,7 @@ public:
           currency = file->security((*it_account).currencyId());
         }
 
-        for (int f = beginDay; f <= m_forecast.forecastDays(); f += m_forecast.accountsCycle()) {
+        for (auto f = beginDay; f <= m_forecast.forecastDays(); f += m_forecast.accountsCycle()) {
           forecastBalance = m_forecast.forecastBalance(*it_account, QDate::currentDate().addDays(f));
           QString amount;
           amount = MyMoneyUtils::formatMoney(forecastBalance, *it_account, currency);
@@ -1116,8 +1122,6 @@ public:
             && (dropMinimum < dropZero
                 || dropZero == -1)) {
           switch (dropMinimum) {
-            case -1:
-              break;
             case 0:
               msg = i18n("The balance of %1 is below the minimum balance %2 today.", (*it_account).name(), MyMoneyUtils::formatMoney(minBalance, *it_account, currency));
               msg = showColoredAmount(msg, true);
@@ -1248,7 +1252,7 @@ public:
       qStableSort(assets.begin(), assets.end(), accountNameLess);
       qStableSort(liabilities.begin(), liabilities.end(), accountNameLess);
       QString statusHeader;
-      if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) {
+      if (KMyMoneySettings::showBalanceStatusOfOnlineAccounts()) {
         QString pathStatusHeader;
         pathStatusHeader = QPixmapToDataUri(Icons::get(Icon::ViewOutbox).pixmap(QSize(16,16)));
         statusHeader = QString("<img src=\"%1\" border=\"0\">").arg(pathStatusHeader);
@@ -1262,7 +1266,7 @@ public:
 
       m_html += "<tr class=\"item\">";
 
-      if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) {
+      if (KMyMoneySettings::showBalanceStatusOfOnlineAccounts()) {
         m_html += "<td class=\"setcolor\">";
         m_html += statusHeader;
         m_html += "</td>";
@@ -1272,15 +1276,18 @@ public:
       m_html += i18n("Asset Accounts");
       m_html += "</td>";
 
-      if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions())
+      if (KMyMoneySettings::showCountOfUnmarkedTransactions())
         m_html += "<td class=\"setcolor\">!M</td>";
 
-      if (KMyMoneyGlobalSettings::showCountOfClearedTransactions())
+      if (KMyMoneySettings::showCountOfClearedTransactions())
         m_html += "<td class=\"setcolor\">C</td>";
 
-      if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
+      if (KMyMoneySettings::showCountOfNotReconciledTransactions())
         m_html += "<td class=\"setcolor\">!R</td>";
 
+      if (KMyMoneySettings::showDateOfLastReconciliation())
+        m_html += "<td class=\"setcolor\">" + i18n("Last Reconciled") + "</td>";
+
       m_html += "<td width=\"15%\" class=\"right\">";
       m_html += i18n("Current Balance");
       m_html += "</td>";
@@ -1288,7 +1295,7 @@ public:
       //intermediate row to separate both columns
       m_html += "<td width=\"10%\" class=\"setcolor\"></td>";
 
-      if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) {
+      if (KMyMoneySettings::showBalanceStatusOfOnlineAccounts()) {
         m_html += "<td class=\"setcolor\">";
         m_html += statusHeader;
         m_html += "</td>";
@@ -1298,24 +1305,28 @@ public:
       m_html += i18n("Liability Accounts");
       m_html += "</td>";
 
-      if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions())
+      if (KMyMoneySettings::showCountOfUnmarkedTransactions())
         m_html += "<td class=\"setcolor\">!M</td>";
 
-      if (KMyMoneyGlobalSettings::showCountOfClearedTransactions())
+      if (KMyMoneySettings::showCountOfClearedTransactions())
         m_html += "<td class=\"setcolor\">C</td>";
 
-      if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions())
+      if (KMyMoneySettings::showCountOfNotReconciledTransactions())
         m_html += "<td class=\"setcolor\">!R</td>";
 
+      if (KMyMoneySettings::showDateOfLastReconciliation())
+        m_html += "<td class=\"setcolor\">" + i18n("Last Reconciled") + "</td>";
+
       m_html += "<td width=\"15%\" class=\"right\">";
       m_html += i18n("Current Balance");
       m_html += "</td></tr>";
 
       QString placeHolder_Status, placeHolder_Counts;
-      if (KMyMoneyGlobalSettings::showBalanceStatusOfOnlineAccounts()) placeHolder_Status = "<td></td>";
-      if (KMyMoneyGlobalSettings::showCountOfUnmarkedTransactions()) placeHolder_Counts = "<td></td>";
-      if (KMyMoneyGlobalSettings::showCountOfClearedTransactions()) placeHolder_Counts += "<td></td>";
-      if (KMyMoneyGlobalSettings::showCountOfNotReconciledTransactions()) placeHolder_Counts += "<td></td>";
+      if (KMyMoneySettings::showBalanceStatusOfOnlineAccounts()) placeHolder_Status = "<td></td>";
+      if (KMyMoneySettings::showCountOfUnmarkedTransactions()) placeHolder_Counts = "<td></td>";
+      if (KMyMoneySettings::showCountOfClearedTransactions()) placeHolder_Counts += "<td></td>";
+      if (KMyMoneySettings::showCountOfNotReconciledTransactions()) placeHolder_Counts += "<td></td>";
+      if (KMyMoneySettings::showDateOfLastReconciliation()) placeHolder_Counts += "<td></td>";
 
       //get asset and liability accounts
       QList<MyMoneyAccount>::const_iterator asset_it = assets.constBegin();
@@ -1334,9 +1345,9 @@ public:
 
           //calculate balance for foreign currency accounts
           if ((*asset_it).currencyId() != file->baseCurrency().id()) {
-            ReportAccount repAcc = ReportAccount((*asset_it).id());
-            MyMoneyMoney curPrice = repAcc.baseCurrencyPrice(QDate::currentDate());
-            MyMoneyMoney baseValue = value * curPrice;
+            const auto curPrice = file->price((*asset_it).tradingCurrencyId(), file->baseCurrency().id(), QDate::currentDate());
+            const auto curRate = curPrice.rate(file->baseCurrency().id());
+            auto baseValue = value * curRate;
             baseValue = baseValue.convert(10000);
             netAssets += baseValue;
           } else {
@@ -1359,9 +1370,9 @@ public:
           value = MyMoneyFile::instance()->balance((*liabilities_it).id(), QDate::currentDate());
           //calculate balance if foreign currency
           if ((*liabilities_it).currencyId() != file->baseCurrency().id()) {
-            ReportAccount repAcc = ReportAccount((*liabilities_it).id());
-            MyMoneyMoney curPrice = repAcc.baseCurrencyPrice(QDate::currentDate());
-            MyMoneyMoney baseValue = value * curPrice;
+            const auto curPrice = file->price((*liabilities_it).tradingCurrencyId(), file->baseCurrency().id(), QDate::currentDate());
+            const auto curRate = curPrice.rate(file->baseCurrency().id());
+            auto baseValue = value * curRate;
             baseValue = baseValue.convert(10000);
             netLiabilities += baseValue;
           } else {
@@ -1414,143 +1425,20 @@ public:
 
   void showBudget()
   {
-    MyMoneyFile* file = MyMoneyFile::instance();
+    m_html += "<div class=\"shadow\"><div class=\"displayblock\"><div class=\"summaryheader\">" + i18n("Budget") + "</div>\n<div class=\"gap\">&nbsp;</div>\n";
+    m_html += "<table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
 
-    if (file->countBudgets()) {
-      int prec = MyMoneyMoney::denomToPrec(file->baseCurrency().smallestAccountFraction());
-      bool isOverrun = false;
-      int i = 0;
-
-      //config report just like "Monthly Budgeted vs Actual
-      MyMoneyReport reportCfg = MyMoneyReport(
-                                  MyMoneyReport::eBudgetActual,
-                                  MyMoneyReport::eMonths,
-                                  TransactionFilter::Date::CurrentMonth,
-                                  MyMoneyReport::eDetailAll,
-                                  i18n("Monthly Budgeted vs. Actual"),
-                                  i18n("Generated Report"));
-
-      reportCfg.setBudget("Any", true);
-
-      reports::PivotTable table(reportCfg);
-
-      PivotGrid grid = table.grid();
-
-      //div header
-      m_html += "<div class=\"shadow\"><div class=\"displayblock\"><div class=\"summaryheader\">" + i18n("Budget") + "</div>\n<div class=\"gap\">&nbsp;</div>\n";
-
-      //display budget summary
-      m_html += "<table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
-      m_html += "<tr class=\"itemtitle\">";
-      m_html += "<td class=\"left\" colspan=\"3\">";
-      m_html += i18n("Current Month Summary");
-      m_html += "</td></tr>";
-      m_html += "<tr class=\"item\">";
-      m_html += "<td class=\"right\" width=\"50%\">";
-      m_html += i18n("Budgeted");
-      m_html += "</td>";
-      m_html += "<td class=\"right\" width=\"20%\">";
-      m_html += i18n("Actual");
-      m_html += "</td>";
-      m_html += "<td class=\"right\" width=\"20%\">";
-      m_html += i18n("Difference");
-      m_html += "</td></tr>";
-
-      m_html += QString("<tr class=\"row-odd\">");
-
-      MyMoneyMoney totalBudgetValue = grid.m_total[eBudget].m_total;
-      MyMoneyMoney totalActualValue = grid.m_total[eActual].m_total;
-      MyMoneyMoney totalBudgetDiffValue = grid.m_total[eBudgetDiff].m_total;
-
-      QString totalBudgetAmount = totalBudgetValue.formatMoney(file->baseCurrency().tradingSymbol(), prec);
-      QString totalActualAmount = totalActualValue.formatMoney(file->baseCurrency().tradingSymbol(), prec);
-      QString totalBudgetDiffAmount = totalBudgetDiffValue.formatMoney(file->baseCurrency().tradingSymbol(), prec);
-
-      m_html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(totalBudgetAmount, totalBudgetValue.isNegative()));
-      m_html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(totalActualAmount, totalActualValue.isNegative()));
-      m_html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(totalBudgetDiffAmount, totalBudgetDiffValue.isNegative()));
-      m_html += "</tr>";
-      m_html += "</table>";
-
-      //budget overrun
-      m_html += "<div class=\"gap\">&nbsp;</div>\n";
-      m_html += "<table width=\"100%\" cellspacing=\"0\" cellpadding=\"2\" class=\"summarytable\" >";
-      m_html += "<tr class=\"itemtitle\">";
-      m_html += "<td class=\"left\" colspan=\"4\">";
-      m_html += i18n("Budget Overruns");
-      m_html += "</td></tr>";
-      m_html += "<tr class=\"item\">";
-      m_html += "<td class=\"left\" width=\"30%\">";
-      m_html += i18n("Account");
-      m_html += "</td>";
-      m_html += "<td class=\"right\" width=\"20%\">";
-      m_html += i18n("Budgeted");
-      m_html += "</td>";
-      m_html += "<td class=\"right\" width=\"20%\">";
-      m_html += i18n("Actual");
-      m_html += "</td>";
-      m_html += "<td class=\"right\" width=\"20%\">";
-      m_html += i18n("Difference");
-      m_html += "</td></tr>";
-
-
-      PivotGrid::iterator it_outergroup = grid.begin();
-      while (it_outergroup != grid.end()) {
-        i = 0;
-        PivotOuterGroup::iterator it_innergroup = (*it_outergroup).begin();
-        while (it_innergroup != (*it_outergroup).end()) {
-          PivotInnerGroup::iterator it_row = (*it_innergroup).begin();
-          while (it_row != (*it_innergroup).end()) {
-            //column number is 1 because the report includes only current month
-            if (it_row.value()[eBudgetDiff].value(1).isNegative()) {
-              //get report account to get the name later
-              ReportAccount rowname = it_row.key();
-
-              //write the outergroup if it is the first row of outergroup being shown
-              if (i == 0) {
-                m_html += "<tr style=\"font-weight:bold;\">";
-                m_html += QString("<td class=\"left\" colspan=\"4\">%1</td>").arg(MyMoneyAccount::accountTypeToString(rowname.accountType()));
-                m_html += "</tr>";
-              }
-              m_html += QString("<tr class=\"row-%1\">").arg(i++ & 0x01 ? "even" : "odd");
-
-              //get values from grid
-              MyMoneyMoney actualValue = it_row.value()[eActual][1];
-              MyMoneyMoney budgetValue = it_row.value()[eBudget][1];
-              MyMoneyMoney budgetDiffValue = it_row.value()[eBudgetDiff][1];
-
-              //format amounts
-              QString actualAmount = actualValue.formatMoney(file->baseCurrency().tradingSymbol(), prec);
-              QString budgetAmount = budgetValue.formatMoney(file->baseCurrency().tradingSymbol(), prec);
-              QString budgetDiffAmount = budgetDiffValue.formatMoney(file->baseCurrency().tradingSymbol(), prec);
-
-              //account name
-              m_html += QString("<td>") + link(VIEW_LEDGER, QString("?id=%1").arg(rowname.id())) + rowname.name() + linkend() + "</td>";
-
-              //show amounts
-              m_html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(budgetAmount, budgetValue.isNegative()));
-              m_html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(actualAmount, actualValue.isNegative()));
-              m_html += QString("<td align=\"right\">%1</td>").arg(showColoredAmount(budgetDiffAmount, budgetDiffValue.isNegative()));
-              m_html += "</tr>";
-
-              //set the flag that there are overruns
-              isOverrun = true;
-            }
-            ++it_row;
-          }
-          ++it_innergroup;
-        }
-        ++it_outergroup;
-      }
-
-      //if no negative differences are found, then inform that
-      if (!isOverrun) {
-        m_html += QString("<tr class=\"row-%1\" style=\"font-weight:bold;\">").arg(i++ & 0x01 ? "even" : "odd");
-        m_html += QString("<td class=\"center\" colspan=\"4\">%1</td>").arg(i18n("No Budget Categories have been overrun"));
-        m_html += "</tr>";
-      }
-      m_html += "</table></div></div>";
+    if (const auto reportsPlugin = pPlugins.data.value(QStringLiteral("reportsview"), nullptr)) {
+      const auto variantReport = reportsPlugin->requestData(QString(), eWidgetPlugin::WidgetType::Budget);
+      if (!variantReport.isNull())
+        m_html.append(variantReport.toString());
+    } else {
+      m_html += QString("<tr>");
+      m_html += QString("<td><center>%1</center></td>").arg(i18n("Enable reports plugin to see this chart."));
+      m_html += QString("</tr>");
     }
+
+    m_html += QString("</table></div></div>");
   }
 
   void showCashFlowSummary()
@@ -1580,7 +1468,7 @@ public:
         //get the splits for each transaction
         foreach (const auto split, transaction.splits()) {
           if (!split.shares().isZero()) {
-            ReportAccount repSplitAcc = ReportAccount(split.accountId());
+            auto repSplitAcc = file->account(split.accountId());
 
             //only add if it is an income or expense
             if (repSplitAcc.isIncomeExpense()) {
@@ -1588,8 +1476,9 @@ public:
 
               //convert to base currency if necessary
               if (repSplitAcc.currencyId() != file->baseCurrency().id()) {
-                MyMoneyMoney curPrice = repSplitAcc.baseCurrencyPrice(transaction.postDate());
-                value = (split.shares() * MyMoneyMoney::MINUS_ONE) * curPrice;
+                const auto curPrice = file->price(repSplitAcc.tradingCurrencyId(), file->baseCurrency().id(), QDate::currentDate());
+                const auto curRate = curPrice.rate(file->baseCurrency().id());
+                value = (split.shares() * MyMoneyMoney::MINUS_ONE) * curRate;
                 value = value.convert(10000);
               } else {
                 value = (split.shares() * MyMoneyMoney::MINUS_ONE);
@@ -1662,13 +1551,13 @@ public:
 
         // take care of the autoCalc stuff
         if ((*sched_it).type() == Schedule::Type::LoanPayment) {
-          QDate nextDate = (*sched_it).nextPayment((*sched_it).lastPayment());
+          nextDate = (*sched_it).nextPayment((*sched_it).lastPayment());
 
           //make sure we have all 'starting balances' so that the autocalc works
           QMap<QString, MyMoneyMoney> balanceMap;
 
           foreach (const auto split, transaction.splits()) {
-            auto acc = file->account(split.accountId());
+            acc = file->account(split.accountId());
             // collect all overdues on the first day
             QDate schedDate = nextDate;
             if (QDate::currentDate() >= nextDate)
@@ -1684,15 +1573,16 @@ public:
         QList<MyMoneySplit>::const_iterator split_it;
         for (split_it = splits.constBegin(); split_it != splits.constEnd(); ++split_it) {
           if ((*split_it).accountId() != acc.id()) {
-            ReportAccount repSplitAcc = ReportAccount((*split_it).accountId());
+            auto repSplitAcc = file->account((*split_it).accountId());
 
             //get the shares and multiply by the quantity of occurrences in the period
             MyMoneyMoney value = (*split_it).shares() * cnt;
 
             //convert to foreign currency if needed
             if (repSplitAcc.currencyId() != file->baseCurrency().id()) {
-              MyMoneyMoney curPrice = repSplitAcc.baseCurrencyPrice(QDate::currentDate());
-              value = value * curPrice;
+              const auto curPrice = file->price(repSplitAcc.tradingCurrencyId(), file->baseCurrency().id(), QDate::currentDate());
+              const auto curRate = curPrice.rate(file->baseCurrency().id());
+              value = value * curRate;
               value = value.convert(10000);
             }
 
@@ -1746,9 +1636,9 @@ public:
               MyMoneyMoney value = MyMoneyFile::instance()->balance((*account_it).id(), QDate::currentDate());
               //calculate balance for foreign currency accounts
               if ((*account_it).currencyId() != file->baseCurrency().id()) {
-                ReportAccount repAcc = ReportAccount((*account_it).id());
-                MyMoneyMoney curPrice = repAcc.baseCurrencyPrice(QDate::currentDate());
-                MyMoneyMoney baseValue = value * curPrice;
+                const auto curPrice = file->price((*account_it).tradingCurrencyId(), file->baseCurrency().id(), QDate::currentDate());
+                const auto curRate = curPrice.rate(file->baseCurrency().id());
+                auto baseValue = value * curRate;
                 liquidAssets += baseValue;
                 liquidAssets = liquidAssets.convert(10000);
               } else {
@@ -1762,9 +1652,9 @@ public:
               value = MyMoneyFile::instance()->balance((*account_it).id(), QDate::currentDate());
               //calculate balance if foreign currency
               if ((*account_it).currencyId() != file->baseCurrency().id()) {
-                ReportAccount repAcc = ReportAccount((*account_it).id());
-                MyMoneyMoney curPrice = repAcc.baseCurrencyPrice(QDate::currentDate());
-                MyMoneyMoney baseValue = value * curPrice;
+                const auto curPrice = file->price((*account_it).tradingCurrencyId(), file->baseCurrency().id(), QDate::currentDate());
+                const auto curRate = curPrice.rate(file->baseCurrency().id());
+                auto baseValue = value * curRate;
                 liquidLiabilities += baseValue;
                 liquidLiabilities = liquidLiabilities.convert(10000);
               } else {
@@ -1956,6 +1846,8 @@ public:
     */
   QSize           m_netWorthGraphLastValidSize;
 
+  QMap< QString, QVector<int> > m_transactionStats;
+
   /**
     * daily forecast balance of accounts
     */
