@@ -32,7 +32,7 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
-#include "kmymoneyglobalsettings.h"
+#include "kmymoneysettings.h"
 #include "mymoneyexception.h"
 #include "knewbankdlg.h"
 #include "menuenums.h"
@@ -42,6 +42,9 @@ using namespace Icons;
 KInstitutionsView::KInstitutionsView(QWidget *parent) :
     KMyMoneyAccountsViewBase(*new KInstitutionsViewPrivate(this), parent)
 {
+  Q_D(KInstitutionsView);
+  d->ui->setupUi(this);
+
   connect(pActions[eMenu::Action::NewInstitution],    &QAction::triggered, this, &KInstitutionsView::slotNewInstitution);
   connect(pActions[eMenu::Action::EditInstitution],   &QAction::triggered, this, &KInstitutionsView::slotEditInstitution);
   connect(pActions[eMenu::Action::DeleteInstitution], &QAction::triggered, this, &KInstitutionsView::slotDeleteInstitution);
@@ -51,10 +54,27 @@ KInstitutionsView::~KInstitutionsView()
 {
 }
 
-void KInstitutionsView::setDefaultFocus()
+void KInstitutionsView::executeCustomAction(eView::Action action)
 {
-  Q_D(KInstitutionsView);
-  QTimer::singleShot(0, d->ui->m_accountTree, SLOT(setFocus()));
+  switch(action) {
+    case eView::Action::Refresh:
+      refresh();
+      break;
+
+    case eView::Action::SetDefaultFocus:
+      {
+        Q_D(KInstitutionsView);
+        QTimer::singleShot(0, d->ui->m_accountTree, SLOT(setFocus()));
+      }
+      break;
+
+    case eView::Action::EditInstitution:
+      slotEditInstitution();
+      break;
+
+    default:
+      break;
+  }
 }
 
 void KInstitutionsView::refresh()
@@ -67,8 +87,8 @@ void KInstitutionsView::refresh()
   d->m_needsRefresh = false;
 
   d->m_proxyModel->invalidate();
-  d->m_proxyModel->setHideEquityAccounts(!KMyMoneyGlobalSettings::expertMode());
-  d->m_proxyModel->setHideClosedAccounts(KMyMoneyGlobalSettings::hideClosedAccounts());
+  d->m_proxyModel->setHideEquityAccounts(!KMyMoneySettings::expertMode());
+  d->m_proxyModel->setHideClosedAccounts(KMyMoneySettings::hideClosedAccounts());
 }
 
 void KInstitutionsView::showEvent(QShowEvent * event)
@@ -77,7 +97,7 @@ void KInstitutionsView::showEvent(QShowEvent * event)
   if (!d->m_proxyModel)
     d->init();
 
-  emit aboutToShow(View::Institutions);
+  emit customActionRequested(View::Institutions, eView::Action::AboutToShow);
 
   if (d->m_needsRefresh)
     refresh();
@@ -124,7 +144,7 @@ void KInstitutionsView::slotNewInstitution()
       ft.commit();
 
     } catch (const MyMoneyException &e) {
-      KMessageBox::information(this, i18n("Cannot add institution: %1", e.what()));
+      KMessageBox::information(this, i18n("Cannot add institution: %1", QString::fromLatin1(e.what())));
     }
   }
   delete dlg;
@@ -157,15 +177,43 @@ void KInstitutionsView::slotEditInstitution()
       try {
         file->modifyInstitution(dlg->institution());
         ft.commit();
-        emit objectSelected(dlg->institution());
+        emit selectByObject(dlg->institution(), eView::Intent::None);
       } catch (const MyMoneyException &e) {
-        KMessageBox::information(this, i18n("Unable to store institution: %1", e.what()));
+        KMessageBox::information(this, i18n("Unable to store institution: %1", QString::fromLatin1(e.what())));
       }
     }
     delete dlg;
 
   } catch (const MyMoneyException &e) {
-    KMessageBox::information(this, i18n("Unable to edit institution: %1", e.what()));
+    KMessageBox::information(this, i18n("Unable to edit institution: %1", QString::fromLatin1(e.what())));
+  }
+}
+
+void KInstitutionsView::slotSelectByObject(const MyMoneyObject& obj, eView::Intent intent)
+{
+  switch(intent) {
+    case eView::Intent::UpdateActions:
+      updateActions(obj);
+      break;
+
+    case eView::Intent::OpenContextMenu:
+      slotShowInstitutionsMenu(static_cast<const MyMoneyInstitution&>(obj));
+      break;
+
+    default:
+      break;
+  }
+}
+
+void KInstitutionsView::slotSelectByVariant(const QVariantList& variant, eView::Intent intent)
+{
+  switch (intent) {
+    case eView::Intent::UpdateNetWorth:
+      if (variant.count() == 1)
+        slotNetWorthChanged(variant.first().value<MyMoneyMoney>());
+      break;
+    default:
+      break;
   }
 }
 
@@ -181,12 +229,12 @@ void KInstitutionsView::slotDeleteInstitution()
 
     try {
       file->removeInstitution(institution);
-      emit objectSelected(MyMoneyInstitution());
+      emit selectByObject(MyMoneyInstitution(), eView::Intent::None);
       ft.commit();
     } catch (const MyMoneyException &e) {
-      KMessageBox::information(this, i18n("Unable to delete institution: %1", e.what()));
+      KMessageBox::information(this, i18n("Unable to delete institution: %1", QString::fromLatin1(e.what())));
     }
   } catch (const MyMoneyException &e) {
-    KMessageBox::information(this, i18n("Unable to delete institution: %1", e.what()));
+    KMessageBox::information(this, i18n("Unable to delete institution: %1", QString::fromLatin1(e.what())));
   }
 }
