@@ -28,12 +28,16 @@
 // ----------------------------------------------------------------------------
 // Project Includes
 
+#include "ui_editintrowizardpage.h"
 
 EditIntroWizardPage::EditIntroWizardPage(QWidget *parent)
-    : EditIntroWizardPageDecl(parent)
+  : QWizardPage(parent),
+    ui(new Ui::EditIntroWizardPage)
 {
+  ui->setupUi(this);
 }
 
-
-#include "editintrowizardpage.moc"
-
+EditIntroWizardPage::~EditIntroWizardPage()
+{
+  delete ui;
+}
